@@ -1,5 +1,6 @@
 """Matchbox."""
 
+from contextlib import suppress
 from importlib.metadata import PackageNotFoundError, version
 
 from matchbox.common.exceptions import MatchboxClientSettingsException
@@ -14,8 +15,5 @@ except MatchboxClientSettingsException:
         "Please ignore if running in server mode. Otherwise, check your .env file.",
     )
 
-try:
+with suppress(PackageNotFoundError):
     __version__ = version("matchbox-db")
-except PackageNotFoundError:
-    # package is not installed
-    pass
