@@ -1,29 +1,34 @@
 """Factory functions for generating model testkits and data for testing."""
 
+import json
 import warnings
 from collections import Counter
+from collections.abc import Hashable
 from functools import cache
 from textwrap import dedent
-from typing import Any, Hashable, Literal, TypeVar
-from unittest.mock import Mock, PropertyMock, create_autospec
+from typing import Any, TypeVar
 
 import numpy as np
+import polars as pl
 import pyarrow as pa
-import pyarrow.compute as pc
 import rustworkx as rx
 from faker import Faker
-from pandas import DataFrame
+from pyarrow import compute as pc
 from pydantic import BaseModel, ConfigDict, model_validator
 from sqlalchemy import create_engine
 
-from matchbox.client.models.dedupers.base import Deduper
-from matchbox.client.models.linkers.base import Linker
+from matchbox.client.dags import DAG
+from matchbox.client.models import add_model_class
+from matchbox.client.models.dedupers.base import Deduper, DeduperSettings
+from matchbox.client.models.linkers.base import Linker, LinkerSettings
 from matchbox.client.models.models import Model
-from matchbox.client.results import Results
+from matchbox.client.queries import Query
 from matchbox.common.arrow import SCHEMA_RESULTS
 from matchbox.common.dtos import (
-    ModelConfig,
+    ModelResolutionName,
+    ModelResolutionPath,
     ModelType,
+    SourceResolutionName,
 )
 from matchbox.common.factories.entities import (
     ClusterEntity,
@@ -38,22 +43,45 @@ from matchbox.common.factories.sources import (
     SourceTestkitParameters,
     linked_sources_factory,
 )
-from matchbox.common.graph import (
-    ModelResolutionName,
-    ResolutionName,
-    SourceResolutionName,
-)
 from matchbox.common.transform import DisjointSet, graph_results
 
 T = TypeVar("T", bound=Hashable)
 
 
-def component_report(all_nodes: list[Any], table: pa.Table) -> dict:
+class MockDeduper(Deduper):
+    """Mock deduper that does nothing."""
+
+    def prepare(self, left: pl.DataFrame) -> None:
+        """Mock prepare method."""
+        return self
+
+    def dedupe(self, left: pl.DataFrame) -> pl.DataFrame:
+        """Mock dedupe method."""
+        return pl.from_arrow(pa.Table.from_pylist([], schema=SCHEMA_RESULTS))
+
+
+class MockLinker(Linker):
+    """Mock linker that does nothing."""
+
+    def prepare(self, left: pl.DataFrame, right: pl.DataFrame) -> None:
+        """Mock prepare method."""
+        return self
+
+    def link(self, left: pl.DataFrame, right: pl.DataFrame) -> pl.DataFrame:
+        """Mock link method."""
+        return pl.from_arrow(pa.Table.from_pylist([], schema=SCHEMA_RESULTS))
+
+
+add_model_class(MockDeduper)
+add_model_class(MockLinker)
+
+
+def component_report(all_nodes: list[Any], table: pl.DataFrame) -> dict:
     """Fast reporting on connected components using rustworkx.
 
     Args:
         all_nodes: list of identities of inputs being matched
-        table: PyArrow table with 'left', 'right' columns
+        table: Polars dataframe with 'left', 'right' columns
 
     Returns:
         dictionary containing basic component statistics
@@ -224,7 +252,7 @@ def generate_dummy_probabilities(
     num_components: int,
     total_rows: int | None = None,
     seed: int = 42,
-) -> pa.Table:
+) -> pl.DataFrame:
     """Generate dummy Arrow probabilities data with guaranteed isolated components.
 
     While much of the factory system uses generate_entity_probabilities, this function
@@ -241,7 +269,7 @@ def generate_dummy_probabilities(
         seed: Random seed for reproducibility
 
     Returns:
-        PyArrow Table with 'left_id', 'right_id', and 'probability' columns
+        Polars dataframe with 'left_id', 'right_id', and 'probability' columns
     """
     # Validate inputs
     deduplicate = False
@@ -391,14 +419,9 @@ def generate_dummy_probabilities(
     # Convert to arrays
     lefts, rights, probs = zip(*all_edges, strict=True)
 
-    # Create PyArrow arrays
-    left_array = pa.array(lefts, type=pa.uint64())
-    right_array = pa.array(rights, type=pa.uint64())
-    prob_array = pa.array(probs, type=pa.uint8())
-
-    return pa.table(
-        [left_array, right_array, prob_array],
-        names=["left_id", "right_id", "probability"],
+    return pl.DataFrame(
+        {"left_id": lefts, "right_id": rights, "probability": probs},
+        schema={"left_id": pl.UInt64, "right_id": pl.UInt64, "probability": pl.UInt8},
     )
 
 
@@ -408,7 +431,7 @@ def generate_entity_probabilities(
     source_entities: frozenset[SourceEntity],
     prob_range: tuple[float, float] = (0.8, 1.0),
     seed: int = 42,
-) -> pa.Table:
+) -> pl.DataFrame:
     """Generate probabilities that will recover entity relationships.
 
     Compares ClusterEntity objects against ground truth SourceEntities by checking
@@ -490,9 +513,11 @@ def generate_entity_probabilities(
             for right_entity in right_group:
                 # For deduplication, only include each pair once
                 # and ensure left_id < right_id
-                if right_entities == left_entities:
-                    if left_entity.id >= right_entity.id:
-                        continue
+                if (
+                    right_entities == left_entities
+                    and left_entity.id >= right_entity.id
+                ):
+                    continue
 
                 # Generate random probability in range
                 prob = rng.integers(prob_min, prob_max + 1)
@@ -500,27 +525,9 @@ def generate_entity_probabilities(
 
     # If no edges were generated, return empty table with correct schema
     if not edges:
-        return pa.table(
-            [
-                pa.array([], type=pa.uint64()),
-                pa.array([], type=pa.uint64()),
-                pa.array([], type=pa.uint8()),
-            ],
-            schema=SCHEMA_RESULTS,
-        )
+        return pl.DataFrame(schema=pl.Schema(SCHEMA_RESULTS))
 
-    # Convert to arrays
-    lefts, rights, probs = zip(*edges, strict=False)
-
-    # Create PyArrow arrays
-    left_array = pa.array(lefts, type=pa.uint64())
-    right_array = pa.array(rights, type=pa.uint64())
-    prob_array = pa.array(probs, type=pa.uint8())
-
-    return pa.table(
-        [left_array, right_array, prob_array],
-        schema=SCHEMA_RESULTS,
-    )
+    return pl.DataFrame(edges, orient="row", schema=pl.Schema(SCHEMA_RESULTS))
 
 
 class ModelTestkit(BaseModel):
@@ -529,11 +536,13 @@ class ModelTestkit(BaseModel):
     model_config = ConfigDict(arbitrary_types_allowed=True)
 
     model: Model
-    left_query: pa.Table
+    left_data: pa.Table
+    left_query: Query
     left_clusters: dict[int, ClusterEntity]
-    right_query: pa.Table | None
+    right_data: pa.Table | None
+    right_query: Query | None
     right_clusters: dict[int, ClusterEntity] | None
-    probabilities: pa.Table
+    probabilities: pl.DataFrame
 
     _entities: tuple[ClusterEntity, ...]
     _threshold: int
@@ -542,7 +551,35 @@ class ModelTestkit(BaseModel):
     @property
     def name(self) -> str:
         """Return the full name of the Model."""
-        return self.model.model_config.name
+        return self.model.name
+
+    @property
+    def resolution_path(self) -> ModelResolutionPath:
+        """Returns the model resolution path."""
+        return self.model.resolution_path
+
+    @property
+    def data(self) -> pa.Table:
+        """Return a PyArrow table in the same format as matchbox queries."""
+        if self.model.config.type == ModelType.DEDUPER:
+            data = self.left_data
+        else:
+            data = pa.concat_tables(
+                [self.left_data, self.right_data], promote_options="default"
+            )
+
+        indices = pc.index_in(data["id"], self._query_lookup["id"])
+        indices = indices.combine_chunks()
+        replacements = pc.take(self._query_lookup["new_id"], indices).combine_chunks()
+
+        new_ids = pc.replace_with_mask(
+            data["id"],
+            pc.is_valid(indices),
+            replacements,
+        )
+
+        id_index = data.schema.get_field_index("id")
+        return data.set_column(id_index, "id", new_ids)
 
     @property
     def entities(self) -> tuple[ClusterEntity, ...]:
@@ -591,72 +628,47 @@ class ModelTestkit(BaseModel):
         self._entities = entities
         self._threshold = value
 
+    def into_dag(self) -> dict:
+        """Turn model into kwargs for `dag.model()`, detaching from original DAG."""
+        return {
+            "name": self.model.name,
+            "model_class": self.model.config.model_class,
+            "model_settings": json.loads(self.model.config.model_settings),
+            "left_query": self.model.left_query,
+            "right_query": self.model.right_query,
+            "truth": self.model.truth,
+            "description": self.model.description,
+        }
+
     @model_validator(mode="after")
     def init_query_lookup(self) -> "ModelTestkit":
         """Initialize the query lookup table."""
         self.threshold = 0
         return self
 
-    @property
-    def mock(self) -> Mock:
-        """Create a mock Model object with this testkit's configuration."""
-        mock_model = create_autospec(Model)
 
-        # Set basic attributes
-        mock_model.model_config = self.model
-        mock_model.left_data = DataFrame()  # Default empty DataFrame
-        mock_model.right_data = (
-            DataFrame() if self.model.type == ModelType.LINKER else None
+def _testkit_to_query(testkit: SourceTestkit | ModelTestkit) -> Query:
+    if isinstance(testkit, SourceTestkit):
+        return Query(testkit.source, dag=testkit.source.dag)
+    else:
+        all_sources = list(testkit.model.left_query.sources)
+        if testkit.model.right_query is not None:
+            all_sources += list(testkit.model.right_query.sources)
+        return Query(
+            *all_sources,
+            model=testkit.model,
+            dag=testkit.model.dag,
         )
 
-        # Mock results property
-        mock_results = Results(probabilities=self.data, metadata=self.model)
-        type(mock_model).results = PropertyMock(return_value=mock_results)
-
-        # Mock run method
-        mock_model.run.return_value = mock_results
-
-        # Mock the model instance based on type
-        if self.model.type == ModelType.LINKER:
-            mock_model.model_instance = create_autospec(Linker)
-            mock_model.model_instance.link.return_value = self.data
-        else:
-            mock_model.model_instance = create_autospec(Deduper)
-            mock_model.model_instance.dedupe.return_value = self.data
-
-        return mock_model
-
-    @property
-    def query(self) -> pa.Table:
-        """Return a PyArrow table in the same format at matchbox.query()."""
-        if self.model.model_config.type == ModelType.DEDUPER:
-            query = self.left_query
-        else:
-            query = pa.concat_tables(
-                [self.left_query, self.right_query], promote_options="default"
-            )
-
-        indices = pc.index_in(query["id"], self._query_lookup["id"])
-        indices = indices.combine_chunks()
-        replacements = pc.take(self._query_lookup["new_id"], indices).combine_chunks()
-
-        new_ids = pc.replace_with_mask(
-            query["id"],
-            pc.is_valid(indices),
-            replacements,
-        )
-
-        id_index = query.schema.get_field_index("id")
-        return query.set_column(id_index, "id", new_ids)
-
 
 def model_factory(
     name: ModelResolutionName | None = None,
+    dag: DAG | None = None,
     description: str | None = None,
     left_testkit: SourceTestkit | ModelTestkit | None = None,
     right_testkit: SourceTestkit | ModelTestkit | None = None,
     true_entities: tuple[SourceEntity, ...] | None = None,
-    model_type: Literal["deduper", "linker"] | None = None,
+    model_type: ModelType | None = None,
     n_true_entities: int | None = None,
     prob_range: tuple[float, float] = (0.8, 1.0),
     seed: int = 42,
@@ -671,6 +683,8 @@ def model_factory(
 
     Args:
         name: Name of the model
+        dag: DAG containing this model.
+            Overridden by dag of left testkit if present.
         description: Description of the model
         left_testkit: A SourceTestkit or ModelTestkit for the left source
         right_testkit: If creating a linker, a SourceTestkit or ModelTestkit for the
@@ -718,22 +732,34 @@ def model_factory(
     # ==== SourceConfig configuration ====
     if left_testkit is not None:
         # Using provided sources
-        left_resolution = left_testkit.name
-        left_query = left_testkit.query
+        left_data = left_testkit.data
+        left_query = _testkit_to_query(left_testkit)
         left_entities = left_testkit.entities
+        if isinstance(left_testkit, SourceTestkit):
+            dag = left_testkit.source.dag
+        else:
+            dag = left_testkit.model.dag
+
+        right_data = None
+        right_query = None
+        right_entities = None
 
         if right_testkit is not None:
             model_type = ModelType.LINKER
-            right_resolution = right_testkit.name
-            right_query = right_testkit.query
+            right_data = right_testkit.data
+            right_query = _testkit_to_query(right_testkit)
             right_entities = right_testkit.entities
         else:
             model_type = ModelType.DEDUPER
-            right_resolution = right_query = right_entities = None
+            right_entities = None
     else:
+        if dag is None:
+            dag = DAG("collection")
+            dag.run = 1
+
         # Create default sources
         engine = create_engine("sqlite:///:memory:")
-        resolved_model_type = ModelType(model_type.lower() if model_type else "deduper")
+        resolved_model_type = model_type or ModelType.DEDUPER
 
         # Define common features
         features = {
@@ -753,7 +779,6 @@ def model_factory(
         }
 
         # Configure left source
-        left_resolution = generator.unique.word()
         left_parameters = SourceTestkitParameters(
             name="crn",
             engine=engine,
@@ -772,7 +797,6 @@ def model_factory(
         # Configure sources based on model type
         source_parameters = [left_parameters]
         if resolved_model_type == ModelType.LINKER:
-            right_resolution = generator.unique.word()
             source_parameters.append(
                 SourceTestkitParameters(
                     name="cdms",
@@ -780,41 +804,44 @@ def model_factory(
                     repetition=1,
                 )
             )
-        else:
-            right_resolution = right_query = right_entities = None
 
         # Generate linked sources
         linked = linked_sources_factory(
+            dag=dag,
             source_parameters=tuple(source_parameters),
             n_true_entities=n_true_entities,
             seed=seed,
         )
 
         # Extract source data
-        left_query = linked.sources["crn"].query
+        left_data = linked.sources["crn"].data
+        left_query = Query(linked.sources["crn"].source, dag=dag)
         left_entities = linked.sources["crn"].entities
 
+        right_data = None
+        right_query = None
+        right_entities = None
         if resolved_model_type == ModelType.LINKER:
-            right_query = linked.sources["cdms"].query
+            right_data = linked.sources["cdms"].data
+            right_query = Query(linked.sources["cdms"].source, dag=dag)
             right_entities = linked.sources["cdms"].entities
 
         dummy_true_entities = tuple(linked.true_entities)
         model_type = resolved_model_type
 
     # ==== Model creation ====
-    metadata = ModelConfig(
+    model_class = MockLinker if model_type == ModelType.LINKER else MockDeduper
+    model_settings = (
+        LinkerSettings() if model_type == ModelType.LINKER else DeduperSettings()
+    )
+    model = Model(
+        dag=dag,
         name=name or generator.unique.word(),
         description=description or generator.sentence(),
-        type=model_type,
-        left_resolution=left_resolution,
-        right_resolution=right_resolution,
-    )
-
-    model = Model(
-        metadata=metadata,
-        model_instance=Mock(),
-        left_data=left_query,
-        right_data=right_query,
+        model_class=model_class,
+        model_settings=model_settings,
+        left_query=left_query,
+        right_query=right_query,
     )
 
     # ==== Entity and probability generation ====
@@ -841,8 +868,10 @@ def model_factory(
     # ==== Final model creation ====
     return ModelTestkit(
         model=model,
+        left_data=left_data,
         left_query=left_query,
         left_clusters={entity.id: entity for entity in left_entities},
+        right_data=right_data,
         right_query=right_query,
         right_clusters={entity.id: entity for entity in right_entities}
         if right_entities
@@ -852,14 +881,14 @@ def model_factory(
 
 
 def query_to_model_factory(
-    left_resolution: ResolutionName,
-    left_query: pa.Table,
+    left_query: Query,
+    left_data: pa.Table,
     left_keys: dict[SourceResolutionName, str],
     true_entities: tuple[SourceEntity, ...],
     name: ModelResolutionName | None = None,
     description: str | None = None,
-    right_resolution: ResolutionName | None = None,
-    right_query: pa.Table | None = None,
+    right_query: Query | None = None,
+    right_data: pa.Table | None = None,
     right_keys: dict[SourceResolutionName, str] | None = None,
     prob_range: tuple[float, float] = (0.8, 1.0),
     seed: int = 42,
@@ -867,16 +896,16 @@ def query_to_model_factory(
     """Turns raw queries from Matchbox into ModelTestkits.
 
     Args:
-        left_resolution: Name of the resolution used for the left query
-        left_query: PyArrow table with left query data
+        left_query: Query generating left data
+        left_data: PyArrow table with left query data
         left_keys: Dictionary mapping source resolution names to key field names
             in left query
         true_entities: Ground truth SourceEntity objects to use for generating
             probabilities
         name: Name of the model
         description: Description of the model
-        right_resolution: Name of the resolution used for the right query
-        right_query: PyArrow table with right query data, if creating a linker
+        right_query: Query generating right data
+        right_data: PyArrow table with right query data, if creating a linker
         right_keys: Dictionary mapping source resolution names to key field names
             in right query
         prob_range: Range of probabilities to generate
@@ -889,46 +918,44 @@ def query_to_model_factory(
     if not (0 <= prob_range[0] <= prob_range[1] <= 1):
         raise ValueError("Probabilities must be increasing values between 0 and 1")
 
+    dag = left_query.dag
+    if right_query and right_query.dag != dag:
+        raise ValueError("DAGs for left and right query must match.")
+
     # Check if right-side arguments are consistently provided
-    right_args = [right_resolution, right_query, right_keys]
+    right_args = [right_query, right_data, right_keys]
     if any(arg is not None for arg in right_args) and not all(
         arg is not None for arg in right_args
     ):
         raise ValueError(
-            "When providing right-side arguments, all of right_resolution, "
-            "right_query, and right_keys must be provided"
+            "When providing right-side arguments, all of right_query, "
+            "right_data, and right_keys must be provided"
         )
 
     # Create a Faker instance with the seed
     generator = Faker()
     generator.seed_instance(seed)
 
-    # Determine model type based on inputs
-    model_type = ModelType.LINKER if right_query is not None else ModelType.DEDUPER
-
     # Create left and right ClusterEntity sets
-    left_clusters = query_to_cluster_entities(left_query, left_keys)
+    left_clusters = query_to_cluster_entities(left_data, left_keys)
 
-    if right_query is not None and right_keys is not None:
-        right_clusters = query_to_cluster_entities(right_query, right_keys)
+    if right_data is not None and right_keys is not None:
+        right_clusters = query_to_cluster_entities(right_data, right_keys)
     else:
         right_clusters = None
 
     # Create model metadata
-    metadata = ModelConfig(
+    model_class = MockLinker if right_data is not None else MockDeduper
+    model_settings = LinkerSettings() if right_data is not None else DeduperSettings()
+
+    model = Model(
+        dag=dag,
         name=name or generator.unique.word(),
         description=description or generator.sentence(),
-        type=model_type,
-        left_resolution=left_resolution,
-        right_resolution=right_resolution,
-    )
-
-    # Create model instance
-    model = Model(
-        metadata=metadata,
-        model_instance=Mock(),
-        left_data=left_query,
-        right_data=right_query,
+        model_class=model_class,
+        model_settings=model_settings,
+        left_query=left_query,
+        right_query=right_query,
     )
 
     # Generate probabilities
@@ -943,8 +970,10 @@ def query_to_model_factory(
     # Create ModelTestkit
     return ModelTestkit(
         model=model,
+        left_data=left_data,
         left_query=left_query,
         left_clusters={entity.id: entity for entity in left_clusters},
+        right_data=right_data,
         right_query=right_query,
         right_clusters={entity.id: entity for entity in right_clusters}
         if right_clusters
