@@ -1,13 +1,16 @@
 """Scenario factories for creating TestkitDAG scenarios."""
 
+from collections.abc import Callable, Generator
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Any, Callable, Generator, Literal
+from typing import Any, Literal
 
 import pyarrow as pa
+from polars.testing import assert_frame_equal
 from pydantic_settings import BaseSettings, SettingsConfigDict
 from sqlalchemy import Engine
 
+from matchbox.client.queries import Query
 from matchbox.common.factories.dags import TestkitDAG
 from matchbox.common.factories.entities import FeatureConfig, SuffixRule
 from matchbox.common.factories.models import query_to_model_factory
@@ -39,7 +42,7 @@ class DevelopmentSettings(BaseSettings):
         extra="ignore",
         env_prefix="MB__DEV__",
         env_nested_delimiter="__",
-        env_file=Path("environments/development.env"),
+        env_file=Path(".env"),
         env_file_encoding="utf-8",
     )
 
@@ -79,7 +82,7 @@ def _testkitdag_to_location(client: Engine, dag: TestkitDAG) -> None:
     * Updates the client of all sources in the DAG
     """
     for source_testkit in dag.sources.values():
-        source_testkit.write_to_location(client=client, set_client=True)
+        source_testkit.write_to_location(set_client=client)
 
 
 # Scenario builders
@@ -94,18 +97,25 @@ def create_bare_scenario(
     **kwargs: Any,
 ) -> TestkitDAG:
     """Create a bare TestkitDAG scenario."""
-    dag = TestkitDAG()
+    dag_testkit = TestkitDAG()
+
+    # Create collection and run
+    backend.create_collection(name=dag_testkit.dag.name)
+    dag_testkit.dag.run = backend.create_run(collection=dag_testkit.dag.name).run_id
 
     # Create linked sources
     linked = linked_sources_factory(
-        n_true_entities=n_entities, seed=seed, engine=warehouse_engine
+        n_true_entities=n_entities,
+        seed=seed,
+        engine=warehouse_engine,
+        dag=dag_testkit.dag,
     )
-    dag.add_source(linked)
+    dag_testkit.add_linked_sources(linked)
 
     # Write sources to warehouse
-    _testkitdag_to_location(warehouse_engine, dag)
+    _testkitdag_to_location(warehouse_engine, dag_testkit)
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("index")
@@ -118,16 +128,22 @@ def create_index_scenario(
 ) -> TestkitDAG:
     """Create an index TestkitDAG scenario."""
     # First create the bare scenario
-    dag = create_bare_scenario(backend, warehouse_engine, n_entities, seed, **kwargs)
+    dag_testkit = create_bare_scenario(
+        backend, warehouse_engine, n_entities, seed, **kwargs
+    )
 
     # Index sources in backend
-    for source_testkit in dag.sources.values():
-        backend.index(
-            source_config=source_testkit.source_config,
+    for source_testkit in dag_testkit.sources.values():
+        backend.create_resolution(
+            resolution=source_testkit.source.to_resolution(),
+            path=source_testkit.resolution_path,
+        )
+        backend.insert_source_data(
+            path=source_testkit.resolution_path,
             data_hashes=source_testkit.data_hashes,
         )
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("dedupe")
@@ -140,38 +156,44 @@ def create_dedupe_scenario(
 ) -> TestkitDAG:
     """Create a dedupe TestkitDAG scenario."""
     # First create the index scenario
-    dag = create_index_scenario(backend, warehouse_engine, n_entities, seed, **kwargs)
+    dag_testkit = create_index_scenario(
+        backend, warehouse_engine, n_entities, seed, **kwargs
+    )
 
-    # Get the linked sources
-    linked_key = next(iter(dag.linked.keys()))
-    linked = dag.linked[linked_key]
+    # Get the LinkedSourcesTestkit using one of the sources
+    linked = dag_testkit.source_to_linked["crn"]
 
     # Create and add deduplication models
-    for testkit in dag.sources.values():
-        source = testkit.source_config
-        name = f"naive_test.{source.name}"
+    for testkit in dag_testkit.sources.values():
+        name = f"naive_test_{testkit.name}"
 
         # Query the raw data
-        source_query = backend.query(source=source.name)
+        source_data = backend.query(source=testkit.resolution_path)
 
         # Build model testkit using query data
         model_testkit = query_to_model_factory(
-            left_resolution=source.name,
-            left_query=source_query,
-            left_keys={source.name: "key"},
+            left_query=Query(testkit.source, dag=dag_testkit.dag),
+            left_data=source_data,
+            left_keys={testkit.name: "key"},
             true_entities=tuple(linked.true_entities),
             name=name,
-            description=f"Deduplication of {source.name}",
+            description=f"Deduplication of {testkit.name}",
             prob_range=(1.0, 1.0),
             seed=seed,
         )
 
         # Add to backend and DAG
-        backend.insert_model(model_config=model_testkit.model.model_config)
-        backend.set_model_results(name=name, results=model_testkit.probabilities)
-        dag.add_model(model_testkit)
+        backend.create_resolution(
+            resolution=model_testkit.model.to_resolution(),
+            path=model_testkit.resolution_path,
+        )
+        backend.insert_model_data(
+            path=model_testkit.resolution_path,
+            results=model_testkit.probabilities.to_arrow(),
+        )
+        dag_testkit.add_model(model_testkit)
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("probabilistic_dedupe")
@@ -184,40 +206,46 @@ def create_probabilistic_dedupe_scenario(
 ) -> TestkitDAG:
     """Create a probabilistic dedupe TestkitDAG scenario."""
     # First create the index scenario
-    dag = create_index_scenario(backend, warehouse_engine, n_entities, seed, **kwargs)
+    dag_testkit = create_index_scenario(
+        backend, warehouse_engine, n_entities, seed, **kwargs
+    )
 
-    # Get the linked sources
-    linked_key = next(iter(dag.linked.keys()))
-    linked = dag.linked[linked_key]
+    # Get the LinkedSourcesTestkit using one of the sources
+    linked = dag_testkit.source_to_linked["crn"]
 
     # Create and add deduplication models
-    for testkit in dag.sources.values():
-        source = testkit.source_config
-        name = f"probabilistic_test.{source.name}"
+    for testkit in dag_testkit.sources.values():
+        name = f"probabilistic_test_{testkit.name}"
 
         # Query the raw data
-        source_query = backend.query(source=source.name)
+        source_data = backend.query(source=testkit.resolution_path)
 
         # Build model testkit using query data
         model_testkit = query_to_model_factory(
-            left_resolution=source.name,
-            left_query=source_query,
-            left_keys={source.name: "key"},
+            left_query=Query(testkit.source, dag=dag_testkit.dag),
+            left_data=source_data,
+            left_keys={testkit.name: "key"},
             true_entities=tuple(linked.true_entities),
             name=name,
-            description=f"Probabilistic deduplication of {source.name}",
+            description=f"Probabilistic deduplication of {testkit.name}",
             prob_range=(0.5, 0.99),
             seed=seed,
         )
         model_testkit.threshold = 50
 
         # Add to backend and DAG
-        backend.insert_model(model_config=model_testkit.model.model_config)
-        backend.set_model_results(name=name, results=model_testkit.probabilities)
-        backend.set_model_truth(name=name, truth=50)
-        dag.add_model(model_testkit)
+        backend.create_resolution(
+            resolution=model_testkit.model.to_resolution(),
+            path=model_testkit.resolution_path,
+        )
+        backend.insert_model_data(
+            path=model_testkit.resolution_path,
+            results=model_testkit.probabilities.to_arrow(),
+        )
+        backend.set_model_truth(path=model_testkit.resolution_path, truth=50)
+        dag_testkit.add_model(model_testkit)
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("link")
@@ -230,30 +258,46 @@ def create_link_scenario(
 ) -> TestkitDAG:
     """Create a link TestkitDAG scenario."""
     # First create the dedupe scenario
-    dag = create_dedupe_scenario(backend, warehouse_engine, n_entities, seed, **kwargs)
+    dag_testkit = create_dedupe_scenario(
+        backend, warehouse_engine, n_entities, seed, **kwargs
+    )
 
-    # Get the linked sources
-    linked_key = next(iter(dag.linked.keys()))
-    linked = dag.linked[linked_key]
+    # Get the LinkedSourcesTestkit using one of the sources
+    linked = dag_testkit.source_to_linked["crn"]
 
     # Extract models for linking
-    crn_model = dag.models["naive_test.crn"]
-    duns_model = dag.models["naive_test.duns"]
-    cdms_model = dag.models["naive_test.cdms"]
+    crn_model = dag_testkit.models["naive_test_crn"]
+    duns_model = dag_testkit.models["naive_test_duns"]
+    cdms_model = dag_testkit.models["naive_test_cdms"]
 
     # Query data for each resolution
-    crn_query = backend.query(source="crn", resolution=crn_model.name)
-    duns_query = backend.query(source="duns", resolution=duns_model.name)
-    cdms_query = backend.query(source="cdms", resolution=cdms_model.name)
+    crn_data = backend.query(
+        source=dag_testkit.sources["crn"].resolution_path,
+        point_of_truth=crn_model.resolution_path,
+    )
+    duns_data = backend.query(
+        source=dag_testkit.sources["duns"].resolution_path,
+        point_of_truth=duns_model.resolution_path,
+    )
+    cdms_data = backend.query(
+        source=dag_testkit.sources["cdms"].resolution_path,
+        point_of_truth=cdms_model.resolution_path,
+    )
 
     # Create CRN-DUNS link
-    crn_duns_name = "deterministic_naive_test.crn_naive_test.duns"
+    crn_duns_name = "deterministic_naive_test_crn_naive_test_duns"
     crn_duns_model = query_to_model_factory(
-        left_resolution=crn_model.name,
-        left_query=crn_query,
+        left_query=Query(
+            dag_testkit.sources["crn"].source,
+            model=crn_model.model,
+            dag=dag_testkit.dag,
+        ),
+        left_data=crn_data,
         left_keys={"crn": "key"},
-        right_resolution=duns_model.name,
-        right_query=duns_query,
+        right_query=Query(
+            dag_testkit.sources["duns"], model=duns_model.model, dag=dag_testkit.dag
+        ),
+        right_data=duns_data,
         right_keys={"duns": "key"},
         true_entities=tuple(linked.true_entities),
         name=crn_duns_name,
@@ -263,18 +307,32 @@ def create_link_scenario(
     )
 
     # Add to backend and DAG
-    backend.insert_model(model_config=crn_duns_model.model.model_config)
-    backend.set_model_results(name=crn_duns_name, results=crn_duns_model.probabilities)
-    dag.add_model(crn_duns_model)
+    backend.create_resolution(
+        resolution=crn_duns_model.model.to_resolution(),
+        path=crn_duns_model.resolution_path,
+    )
+    backend.insert_model_data(
+        path=crn_duns_model.resolution_path,
+        results=crn_duns_model.probabilities.to_arrow(),
+    )
+    dag_testkit.add_model(crn_duns_model)
 
     # Create CRN-CDMS link
-    crn_cdms_name = "probabilistic_naive_test.crn_naive_test.cdms"
+    crn_cdms_name = "probabilistic_naive_test_crn_naive_test_cdms"
     crn_cdms_model = query_to_model_factory(
-        left_resolution=crn_model.name,
-        left_query=crn_query,
+        left_query=Query(
+            dag_testkit.sources["crn"].source,
+            model=crn_model.model,
+            dag=dag_testkit.dag,
+        ),
+        left_data=crn_data,
         left_keys={"crn": "key"},
-        right_resolution=cdms_model.name,
-        right_query=cdms_query,
+        right_query=Query(
+            dag_testkit.sources["cdms"].source,
+            model=cdms_model.model,
+            dag=dag_testkit.dag,
+        ),
+        right_data=cdms_data,
         right_keys={"cdms": "key"},
         true_entities=tuple(linked.true_entities),
         name=crn_cdms_name,
@@ -282,33 +340,54 @@ def create_link_scenario(
         seed=seed,
     )
 
-    backend.insert_model(model_config=crn_cdms_model.model.model_config)
-    backend.set_model_results(name=crn_cdms_name, results=crn_cdms_model.probabilities)
-    backend.set_model_truth(name=crn_cdms_name, truth=75)
-    dag.add_model(crn_cdms_model)
+    # Add to backend and DAG
+    backend.create_resolution(
+        path=crn_cdms_model.resolution_path,
+        resolution=crn_cdms_model.model.to_resolution(),
+    )
+    backend.insert_model_data(
+        path=crn_cdms_model.resolution_path,
+        results=crn_cdms_model.probabilities.to_arrow(),
+    )
+    backend.set_model_truth(path=crn_cdms_model.resolution_path, truth=75)
+    dag_testkit.add_model(crn_cdms_model)
 
     # Create final join
     # Query the previous link's results
-    crn_cdms_query_crn_only = backend.query(
-        source="crn", resolution=crn_cdms_name
+    crn_cdms_data_crn_only = backend.query(
+        source=dag_testkit.sources["crn"].resolution_path,
+        point_of_truth=crn_cdms_model.resolution_path,
     ).rename_columns(["id", "keys_crn"])
-    crn_cdms_query_cdms_only = backend.query(
-        source="cdms", resolution=crn_cdms_name
+    crn_cdms_data_cdms_only = backend.query(
+        source=dag_testkit.sources["cdms"].resolution_path,
+        point_of_truth=crn_cdms_model.resolution_path,
     ).rename_columns(["id", "keys_cdms"])
-    crn_cdms_query = pa.concat_tables(
-        [crn_cdms_query_crn_only, crn_cdms_query_cdms_only],
+    crn_cdms_data = pa.concat_tables(
+        [crn_cdms_data_crn_only, crn_cdms_data_cdms_only],
         promote_options="default",
     ).combine_chunks()
 
-    duns_query_linked = backend.query(source="duns", resolution=duns_model.name)
+    duns_data_linked = backend.query(
+        source=dag_testkit.sources["duns"].resolution_path,
+        point_of_truth=duns_model.resolution_path,
+    )
 
     final_join_name = "final_join"
     final_join_model = query_to_model_factory(
-        left_resolution=crn_cdms_name,
-        left_query=crn_cdms_query,
+        left_query=Query(
+            dag_testkit.sources["crn"].source,
+            dag_testkit.sources["cdms"].source,
+            model=crn_cdms_model.model,
+            dag=dag_testkit.dag,
+        ),
+        left_data=crn_cdms_data,
         left_keys={"crn": "keys_crn", "cdms": "keys_cdms"},
-        right_resolution=duns_model.name,
-        right_query=duns_query_linked,
+        right_query=Query(
+            dag_testkit.sources["duns"].source,
+            model=duns_model.model,
+            dag=dag_testkit.dag,
+        ),
+        right_data=duns_data_linked,
         right_keys={"duns": "key"},
         true_entities=tuple(linked.true_entities),
         name=final_join_name,
@@ -316,13 +395,18 @@ def create_link_scenario(
         seed=seed,
     )
 
-    backend.insert_model(model_config=final_join_model.model.model_config)
-    backend.set_model_results(
-        name=final_join_name, results=final_join_model.probabilities
+    # Add to backend and DAG
+    backend.create_resolution(
+        resolution=final_join_model.model.to_resolution(),
+        path=final_join_model.resolution_path,
     )
-    dag.add_model(final_join_model)
+    backend.insert_model_data(
+        path=final_join_model.resolution_path,
+        results=final_join_model.probabilities.to_arrow(),
+    )
+    dag_testkit.add_model(final_join_model)
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("alt_dedupe")
@@ -334,7 +418,11 @@ def create_alt_dedupe_scenario(
     **kwargs: Any,
 ) -> TestkitDAG:
     """Create a TestkitDAG scenario with two alternative dedupers."""
-    dag = TestkitDAG()
+    dag_testkit = TestkitDAG()
+
+    # Create collection and run
+    backend.create_collection(name=dag_testkit.dag.name)
+    dag_testkit.dag.run = backend.create_run(collection=dag_testkit.dag.name).run_id
 
     # Create linked sources
     company_name_feature = FeatureConfig(
@@ -350,66 +438,75 @@ def create_alt_dedupe_scenario(
         repetition=1,
     )
 
-    linked = linked_sources_factory(source_parameters=(foo_a_tkit_source,))
-    dag.add_source(linked)
+    linked = linked_sources_factory(
+        source_parameters=(foo_a_tkit_source,), dag=dag_testkit.dag
+    )
+    dag_testkit.add_linked_sources(linked)
 
     # Write sources to warehouse
-    _testkitdag_to_location(warehouse_engine, dag)
+    _testkitdag_to_location(warehouse_engine, dag_testkit)
 
     # Index sources in backend
-    for source_testkit in dag.sources.values():
-        backend.index(
-            source_config=source_testkit.source_config,
+    for source_testkit in dag_testkit.sources.values():
+        backend.create_resolution(
+            resolution=source_testkit.source.to_resolution(),
+            path=source_testkit.resolution_path,
+        )
+        backend.insert_source_data(
+            path=source_testkit.resolution_path,
             data_hashes=source_testkit.data_hashes,
         )
 
     # Create and add deduplication models
-    for testkit in dag.sources.values():
-        source = testkit.source_config
-        model_name1 = f"dedupe.{source.name}"
-        model_name2 = f"dedupe2.{source.name}"
+    for testkit in dag_testkit.sources.values():
+        model_name1 = f"dedupe_{testkit.name}"
+        model_name2 = f"dedupe2_{testkit.name}"
 
         # Query the raw data
-        source_query = backend.query(source=source.name)
+        source_data = backend.query(source=testkit.resolution_path)
 
         # Build model testkit using query data
         model_testkit1 = query_to_model_factory(
-            left_resolution=source.name,
-            left_query=source_query,
-            left_keys={source.name: "key"},
+            left_query=Query(testkit.source, dag=dag_testkit.dag),
+            left_data=source_data,
+            left_keys={testkit.name: "key"},
             true_entities=tuple(linked.true_entities),
             name=model_name1,
-            description=f"Deduplication of {source.name}",
+            description=f"Deduplication of {testkit.name}",
             prob_range=(0.5, 1.0),
             seed=seed,
         )
 
         model_testkit2 = query_to_model_factory(
-            left_resolution=source.name,
-            left_query=source_query,
-            left_keys={source.name: "key"},
+            left_query=Query(testkit.source, dag=dag_testkit.dag),
+            left_data=source_data,
+            left_keys={testkit.name: "key"},
             true_entities=tuple(linked.true_entities),
             name=model_name2,
-            description=f"Deduplication of {source.name}",
+            description=f"Deduplication of {testkit.name}",
             prob_range=(0.5, 1.0),
             seed=seed,
         )
 
-        assert model_testkit1.probabilities.num_rows > 0
-        assert model_testkit1.probabilities == model_testkit2.probabilities
+        assert len(model_testkit1.probabilities) > 0
+        assert_frame_equal(model_testkit1.probabilities, model_testkit2.probabilities)
 
         for model, threshold in ((model_testkit1, 50), (model_testkit2, 75)):
             model.threshold = threshold
 
             # Add both models to backend and DAG
-            backend.insert_model(model_config=model.model.model_config)
-            backend.set_model_results(name=model.name, results=model.probabilities)
-            backend.set_model_truth(name=model.name, truth=threshold)
+            backend.create_resolution(
+                path=model.resolution_path, resolution=model.model.to_resolution()
+            )
+            backend.insert_model_data(
+                path=model.resolution_path, results=model.probabilities.to_arrow()
+            )
+            backend.set_model_truth(path=model.resolution_path, truth=threshold)
 
             # Add to DAG
-            dag.add_model(model)
+            dag_testkit.add_model(model)
 
-    return dag
+    return dag_testkit
 
 
 @register_scenario("convergent")
@@ -426,7 +523,11 @@ def create_convergent_scenario(
     indexed sources with repetition, and two naive dedupe models that haven't yet
     had their results inserted.
     """
-    dag = TestkitDAG()
+    dag_testkit = TestkitDAG()
+
+    # Create collection and run
+    backend.create_collection(name=dag_testkit.dag.name)
+    dag_testkit.dag.run = backend.create_run(collection=dag_testkit.dag.name).run_id
 
     # Create linked sources
     company_name_feature = FeatureConfig(
@@ -446,47 +547,47 @@ def create_convergent_scenario(
         source_parameters=(
             foo_a_tkit_source,
             foo_a_tkit_source.model_copy(update={"name": "foo_b"}),
-        )
+        ),
+        dag=dag_testkit.dag,
     )
 
-    dag.add_source(linked)
+    dag_testkit.add_linked_sources(linked)
 
     # Write sources to warehouse
-    _testkitdag_to_location(warehouse_engine, dag)
+    _testkitdag_to_location(warehouse_engine, dag_testkit)
 
     # Index sources in backend
-    for source_testkit in dag.sources.values():
-        backend.index(
-            source_config=source_testkit.source_config,
+    for source_testkit in dag_testkit.sources.values():
+        backend.create_resolution(
+            resolution=source_testkit.source.to_resolution(),
+            path=source_testkit.resolution_path,
+        )
+        backend.insert_source_data(
+            path=source_testkit.resolution_path,
             data_hashes=source_testkit.data_hashes,
         )
 
-    # Create and add deduplication models
-    for testkit in dag.sources.values():
-        source = testkit.source_config
-        name = f"naive_test.{source.name}"
-
         # Query the raw data
-        source_query = backend.query(source=source.name)
+        source_query = backend.query(source=source_testkit.resolution_path)
 
         # Build model testkit using query data
         model_testkit = query_to_model_factory(
-            left_resolution=source.name,
-            left_query=source_query,
-            left_keys={source.name: "key"},
+            left_query=Query(source_testkit.source, dag=dag_testkit.dag),
+            left_data=source_query,
+            left_keys={source_testkit.name: "key"},
             true_entities=tuple(linked.true_entities),
-            name=name,
-            description=f"Deduplication of {source.name}",
+            name=f"naive_test_{source_testkit.name}",
+            description=f"Deduplication of {source_testkit.name}",
             prob_range=(1.0, 1.0),
             seed=seed,
         )
 
-        assert model_testkit.probabilities.num_rows > 0
+        assert len(model_testkit.probabilities) > 0
 
         # Add to DAG
-        dag.add_model(model_testkit)
+        dag_testkit.add_model(model_testkit)
 
-    return dag
+    return dag_testkit
 
 
 @contextmanager
@@ -518,21 +619,25 @@ def setup_scenario(
     # Check if we have a backend snapshot cached
     if cache_key in _DATABASE_SNAPSHOTS_CACHE:
         # Load cached snapshot and DAG
-        dag, snapshot = _DATABASE_SNAPSHOTS_CACHE[cache_key]
-        dag = dag.model_copy(deep=True)
+        dag_testkit, snapshot = _DATABASE_SNAPSHOTS_CACHE[cache_key]
+        # Because the location client can't be deep-copied, this will reuse an
+        # old engine, with the same URL as `warehouse`, but a different object
+        # Thus, in _testkitdag_to_location we need to overwrite all testkits with
+        # our new warehouse object
+        dag_testkit = dag_testkit.model_copy(deep=True)
 
         # Restore backend and write sources to warehouse
         backend.restore(snapshot=snapshot)
-        _testkitdag_to_location(warehouse, dag)
+        _testkitdag_to_location(warehouse, dag_testkit)
     else:
         # Create new TestkitDAG with proper backend integration
         scenario_builder = SCENARIO_REGISTRY[scenario_type]
-        dag = scenario_builder(backend, warehouse, n_entities, seed, **kwargs)
+        dag_testkit = scenario_builder(backend, warehouse, n_entities, seed, **kwargs)
 
         # Cache the snapshot and DAG
-        _DATABASE_SNAPSHOTS_CACHE[cache_key] = (dag, backend.dump())
+        _DATABASE_SNAPSHOTS_CACHE[cache_key] = (dag_testkit, backend.dump())
 
     try:
-        yield dag
+        yield dag_testkit
     finally:
         backend.clear(certain=True)
