@@ -2,8 +2,10 @@
 
 from datetime import datetime
 from importlib.metadata import version
+from pathlib import Path
 from typing import Annotated
 
+import pyarrow.parquet as pq
 from fastapi import (
     BackgroundTasks,
     Depends,
@@ -16,7 +18,13 @@ from fastapi import (
     status,
 )
 from fastapi.encoders import jsonable_encoder
+from fastapi.openapi.docs import (
+    get_swagger_ui_html,
+    get_swagger_ui_oauth2_redirect_html,
+)
 from fastapi.responses import JSONResponse
+from fastapi.staticfiles import StaticFiles
+from pyarrow import ArrowInvalid
 from starlette.exceptions import HTTPException as StarletteHTTPException
 
 from matchbox.common.arrow import table_to_buffer
@@ -24,22 +32,30 @@ from matchbox.common.dtos import (
     BackendCountableType,
     BackendResourceType,
     BackendUploadType,
+    CollectionName,
     CountResult,
+    CRUDOperation,
     LoginAttempt,
     LoginResult,
+    Match,
+    ModelResolutionName,
     NotFoundError,
     OKMessage,
+    ResolutionPath,
+    ResourceOperationStatus,
+    RunID,
+    SourceResolutionName,
+    SourceResolutionPath,
     UploadStage,
     UploadStatus,
 )
 from matchbox.common.exceptions import (
+    MatchboxCollectionNotFoundError,
     MatchboxDeletionNotConfirmed,
     MatchboxResolutionNotFoundError,
+    MatchboxRunNotFoundError,
     MatchboxServerFileError,
-    MatchboxSourceNotFoundError,
 )
-from matchbox.common.graph import ResolutionGraph, ResolutionName, SourceResolutionName
-from matchbox.common.sources import Match
 from matchbox.server.api.dependencies import (
     BackendDependency,
     ParquetResponse,
@@ -48,35 +64,109 @@ from matchbox.server.api.dependencies import (
     authorisation_dependencies,
     lifespan,
 )
-from matchbox.server.api.routers import eval, models, resolutions, sources
+from matchbox.server.api.routers import collection, eval
 from matchbox.server.uploads import process_upload, process_upload_celery, table_to_s3
 
 app = FastAPI(
     title="matchbox API",
     version=version("matchbox_db"),
     lifespan=lifespan,
+    docs_url=None,
+    redoc_url=None,
 )
-app.include_router(models.router)
-app.include_router(sources.router)
-app.include_router(resolutions.router)
+app.include_router(collection.router)
 app.include_router(eval.router)
 
+static_dir = Path(__file__).parent / "static"
+app.mount("/static", StaticFiles(directory=static_dir), name="static")
+
 
 @app.exception_handler(StarletteHTTPException)
-async def http_exception_handler(request, exc):
+async def http_exception_handler(
+    request: Request, exc: StarletteHTTPException
+) -> JSONResponse:
     """Overwrite the default JSON schema for an `HTTPException`."""
     return JSONResponse(
         content=jsonable_encoder(exc.detail), status_code=exc.status_code
     )
 
 
+@app.exception_handler(MatchboxCollectionNotFoundError)
+async def collection_not_found_handler(
+    request: Request, exc: MatchboxCollectionNotFoundError
+) -> JSONResponse:
+    """Handle collection not found errors."""
+    detail = NotFoundError(
+        details=str(exc), entity=BackendResourceType.COLLECTION
+    ).model_dump()
+    return JSONResponse(
+        status_code=404,
+        content=jsonable_encoder(detail),
+    )
+
+
+@app.exception_handler(MatchboxRunNotFoundError)
+async def run_not_found_handler(
+    request: Request, exc: MatchboxRunNotFoundError
+) -> JSONResponse:
+    """Handle run not found errors."""
+    detail = NotFoundError(
+        details=str(exc), entity=BackendResourceType.RUN
+    ).model_dump()
+    return JSONResponse(
+        status_code=404,
+        content=jsonable_encoder(detail),
+    )
+
+
+@app.exception_handler(MatchboxResolutionNotFoundError)
+async def resolution_not_found_handler(
+    request: Request, exc: MatchboxResolutionNotFoundError
+) -> JSONResponse:
+    """Handle resolution not found errors."""
+    detail = NotFoundError(
+        details=str(exc), entity=BackendResourceType.RESOLUTION
+    ).model_dump()
+    return JSONResponse(
+        status_code=404,
+        content=jsonable_encoder(detail),
+    )
+
+
+@app.exception_handler(MatchboxDeletionNotConfirmed)
+async def deletion_not_confirmed_handler(
+    request: Request, exc: MatchboxDeletionNotConfirmed
+) -> JSONResponse:
+    """Handle deletion not confirmed errors."""
+    # Extract resource name from request
+    path_parts = request.url.path.split("/")
+    resource_name = path_parts[-1] if path_parts else "unknown"
+
+    detail = ResourceOperationStatus(
+        success=False,
+        name=resource_name,
+        operation=CRUDOperation.DELETE,
+        details=str(exc),
+    ).model_dump()
+
+    return JSONResponse(
+        status_code=409,
+        content=jsonable_encoder(detail),
+    )
+
+
 @app.middleware("http")
 async def add_security_headers(request: Request, call_next):
     """Improve security by adding headers to all responses."""
     response: Response = await call_next(request)
     response.headers["Cache-control"] = "no-store, no-cache"
     response.headers["Content-Security-Policy"] = (
-        "default-src 'none'; frame-ancestors 'none'; form-action 'none'; sandbox"
+        # Restrict by default
+        "default-src 'none'; frame-ancestors 'none'; form-action 'none';"
+        # Load Swagger CSS, favicon and openapi.json
+        "style-src 'self'; img-src 'self' data:; connect-src 'self'; "
+        # Load Swagger JS, hard-coding the expected file hash
+        "script-src 'self' 'sha256-QOOQu4W1oxGqd2nbXbxiA1Di6OHQOLQD+o+G9oWL8YY='"
     )
     response.headers["Strict-Transport-Security"] = (
         "max-age=31536000; includeSubDomains"
@@ -130,6 +220,34 @@ def upload_file(
     * Upload is already being processed
     * Uploaded data doesn't match the metadata schema
     """
+    # Validate file
+    if ".parquet" not in file.filename:
+        raise HTTPException(
+            status_code=400,
+            detail=UploadStatus(
+                id=upload_id,
+                update_timestamp=datetime.now(),
+                stage=UploadStage.READY,
+                details=(
+                    f"Server expected .parquet file, got {file.filename.split('.')[-1]}"
+                ),
+            ).model_dump(),
+        )
+
+    # pyarrow validates Parquet magic numbers when loading file
+    try:
+        pq.ParquetFile(file.file)
+    except ArrowInvalid as e:
+        raise HTTPException(
+            status_code=400,
+            detail=UploadStatus(
+                id=upload_id,
+                update_timestamp=datetime.now(),
+                stage=UploadStage.READY,
+                details=(f"Invalid Parquet file: {str(e)}"),
+            ).model_dump(),
+        ) from e
+
     # Get and validate cache entry
     upload_entry = upload_tracker.get(upload_id=upload_id)
     if not upload_entry:
@@ -138,7 +256,7 @@ def upload_file(
             detail=UploadStatus(
                 id=upload_id,
                 update_timestamp=datetime.now(),
-                stage="unknown",
+                stage=UploadStage.UNKNOWN,
                 details=(
                     "Upload ID not found or expired. Entries expire after 30 minutes "
                     "of inactivity, including failed processes."
@@ -185,7 +303,7 @@ def upload_file(
                 tracker=upload_tracker,
                 s3_client=client,
                 upload_type=upload_entry.status.entity,
-                resolution_name=upload_entry.metadata.name,
+                resolution_name=upload_entry.path.name,
                 upload_id=upload_id,
                 bucket=bucket,
                 filename=key,
@@ -193,7 +311,7 @@ def upload_file(
         case "celery":
             process_upload_celery.delay(
                 upload_type=upload_entry.status.entity,
-                resolution_name=upload_entry.metadata.name,
+                resolution_name=upload_entry.path.name,
                 upload_id=upload_id,
                 bucket=bucket,
                 filename=key,
@@ -237,7 +355,7 @@ def get_upload_status(
             status_code=400,
             detail=UploadStatus(
                 id=upload_id,
-                stage="unknown",
+                stage=UploadStage.UNKNOWN,
                 update_timestamp=datetime.now(),
                 details=(
                     "Upload ID not found or expired. Entries expire after 30 minutes "
@@ -259,17 +377,27 @@ def get_upload_status(
 )
 def query(
     backend: BackendDependency,
+    collection: CollectionName,
+    run_id: RunID,
     source: SourceResolutionName,
     return_leaf_id: bool,
-    resolution: ResolutionName | None = None,
+    resolution: ModelResolutionName | None = None,
     threshold: int | None = None,
     limit: int | None = None,
 ) -> ParquetResponse:
     """Query Matchbox for matches based on a source resolution name."""
     try:
         res = backend.query(
-            source=source,
-            resolution=resolution,
+            source=SourceResolutionPath(
+                collection=collection,
+                run=run_id,
+                name=source,
+            ),
+            point_of_truth=ResolutionPath(
+                collection=collection, run=run_id, name=resolution
+            )
+            if resolution
+            else None,
             threshold=threshold,
             return_leaf_id=return_leaf_id,
             limit=limit,
@@ -281,13 +409,6 @@ def query(
                 details=str(e), entity=BackendResourceType.RESOLUTION
             ).model_dump(),
         ) from e
-    except MatchboxSourceNotFoundError as e:
-        raise HTTPException(
-            status_code=404,
-            detail=NotFoundError(
-                details=str(e), entity=BackendResourceType.SOURCE
-            ).model_dump(),
-        ) from e
 
     buffer = table_to_buffer(res)
     return ParquetResponse(buffer.getvalue())
@@ -299,19 +420,32 @@ def query(
 )
 def match(
     backend: BackendDependency,
+    collection: CollectionName,
+    run_id: RunID,
     targets: Annotated[list[SourceResolutionName], Query()],
     source: SourceResolutionName,
     key: str,
-    resolution: ResolutionName,
+    resolution: ModelResolutionName,
     threshold: int | None = None,
 ) -> list[Match]:
     """Match a source key against a list of target source resolutions."""
+    targets = [
+        SourceResolutionPath(collection=collection, run=run_id, name=t) for t in targets
+    ]
     try:
         res = backend.match(
             key=key,
-            source=source,
+            source=SourceResolutionPath(
+                collection=collection,
+                run=run_id,
+                name=source,
+            ),
             targets=targets,
-            resolution=resolution,
+            point_of_truth=ResolutionPath(
+                collection=collection,
+                run=run_id,
+                name=resolution,
+            ),
             threshold=threshold,
         )
     except MatchboxResolutionNotFoundError as e:
@@ -321,13 +455,6 @@ def match(
                 details=str(e), entity=BackendResourceType.RESOLUTION
             ).model_dump(),
         ) from e
-    except MatchboxSourceNotFoundError as e:
-        raise HTTPException(
-            status_code=404,
-            detail=NotFoundError(
-                details=str(e), entity=BackendResourceType.SOURCE
-            ).model_dump(),
-        ) from e
 
     return res
 
@@ -335,12 +462,6 @@ def match(
 # Admin
 
 
-@app.get("/report/resolutions")
-def get_resolutions(backend: BackendDependency) -> ResolutionGraph:
-    """Get the resolution graph."""
-    return backend.get_resolution_graph()
-
-
 @app.get("/database/count")
 def count_backend_items(
     backend: BackendDependency,
@@ -360,7 +481,16 @@ def count_backend_items(
 
 @app.delete(
     "/database",
-    responses={409: {"model": str}},
+    responses={
+        409: {
+            "model": ResourceOperationStatus,
+            **ResourceOperationStatus.status_409_examples(),
+        },
+        500: {
+            "model": ResourceOperationStatus,
+            **ResourceOperationStatus.status_500_examples(),
+        },
+    },
     dependencies=[Depends(authorisation_dependencies)],
 )
 def clear_database(
@@ -373,13 +503,44 @@ def clear_database(
             )
         ),
     ] = False,
-) -> OKMessage:
+) -> ResourceOperationStatus:
     """Delete all data from the backend whilst retaining tables."""
     try:
         backend.clear(certain=certain)
-        return OKMessage()
-    except MatchboxDeletionNotConfirmed as e:
+        return ResourceOperationStatus(
+            success=True, name="database", operation=CRUDOperation.DELETE
+        )
+    except MatchboxDeletionNotConfirmed:
+        raise
+    except Exception as e:
         raise HTTPException(
-            status_code=409,
-            detail=str(e),
+            status_code=500,
+            detail=ResourceOperationStatus(
+                success=False,
+                name="database",
+                operation=CRUDOperation.DELETE,
+                details=str(e),
+            ),
         ) from e
+
+
+# Swagger UI
+
+
+@app.get("/docs", include_in_schema=False)
+async def custom_swagger_ui_html():
+    """Get locally hosted docs."""
+    return get_swagger_ui_html(
+        openapi_url=app.openapi_url,
+        title=app.title + " - Swagger UI",
+        oauth2_redirect_url=app.swagger_ui_oauth2_redirect_url,
+        swagger_js_url="/static/swagger-ui-bundle.js",
+        swagger_css_url="/static/swagger-ui.css",
+        swagger_favicon_url="/static/favicon.png",
+    )
+
+
+@app.get(app.swagger_ui_oauth2_redirect_url, include_in_schema=False)
+async def swagger_ui_redirect():
+    """Helper for OAuth2."""
+    return get_swagger_ui_oauth2_redirect_html()
