@@ -2,16 +2,19 @@
 
 import zipfile
 from io import BytesIO
-from typing import Annotated
 
-from fastapi import APIRouter, HTTPException, Query, Response, status
+from fastapi import APIRouter, HTTPException, Response, status
 
 from matchbox.common.arrow import JudgementsZipFilenames, table_to_buffer
 from matchbox.common.dtos import (
     BackendParameterType,
     BackendResourceType,
+    CollectionName,
     InvalidParameterError,
+    ModelResolutionName,
+    ModelResolutionPath,
     NotFoundError,
+    RunID,
 )
 from matchbox.common.eval import Judgement, ModelComparison
 from matchbox.common.exceptions import (
@@ -21,7 +24,6 @@ from matchbox.common.exceptions import (
     MatchboxTooManySamplesRequested,
     MatchboxUserNotFoundError,
 )
-from matchbox.common.graph import ModelResolutionName
 from matchbox.server.api.dependencies import (
     BackendDependency,
     ParquetResponse,
@@ -81,27 +83,19 @@ def get_judgements(backend: BackendDependency) -> ParquetResponse:
     return ZipResponse(zip_buffer.getvalue())
 
 
-@router.get(
+@router.post(
     "/compare",
     responses={404: {"model": NotFoundError}},
 )
 def compare_models(
     backend: BackendDependency,
-    resolutions: Annotated[
-        list[ModelResolutionName],
-        Query(description="The resolution names for the models to compare."),
-    ],
+    resolutions: list[ModelResolutionPath],
 ) -> ModelComparison:
     """Return comparison of selected models."""
     try:
         return backend.compare_models(resolutions)
-    except MatchboxResolutionNotFoundError as e:
-        raise HTTPException(
-            status_code=404,
-            detail=NotFoundError(
-                details=str(e), entity=BackendResourceType.RESOLUTION
-            ).model_dump(),
-        ) from e
+    except MatchboxResolutionNotFoundError:
+        raise
     except MatchboxNoJudgements as e:
         raise HTTPException(
             status_code=404,
@@ -116,20 +110,26 @@ def compare_models(
     responses={404: {"model": NotFoundError}, 422: {"model": InvalidParameterError}},
 )
 def sample(
-    backend: BackendDependency, n: int, resolution: ModelResolutionName, user_id: int
+    backend: BackendDependency,
+    collection: CollectionName,
+    run_id: RunID,
+    resolution: ModelResolutionName,
+    n: int,
+    user_id: int,
 ) -> ParquetResponse:
     """Sample n cluster to validate."""
     try:
-        sample = backend.sample_for_eval(n=n, resolution=resolution, user_id=user_id)
+        sample = backend.sample_for_eval(
+            path=ModelResolutionPath(
+                collection=collection, run=run_id, name=resolution
+            ),
+            n=n,
+            user_id=user_id,
+        )
         buffer = table_to_buffer(sample)
         return ParquetResponse(buffer.getvalue())
-    except MatchboxResolutionNotFoundError as e:
-        raise HTTPException(
-            status_code=404,
-            detail=NotFoundError(
-                details=str(e), entity=BackendResourceType.RESOLUTION
-            ).model_dump(),
-        ) from e
+    except MatchboxResolutionNotFoundError:
+        raise
     except MatchboxUserNotFoundError as e:
         raise HTTPException(
             status_code=404,
