@@ -3,33 +3,34 @@
 from itertools import chain
 from typing import TYPE_CHECKING, Any, ParamSpec, TypeVar
 
+import polars as pl
 from pyarrow import Table
 from pydantic import BaseModel
-from sqlalchemy import and_, bindparam, delete, func, or_, select
+from sqlalchemy import and_, bindparam, delete, func, or_, select, update
 
 from matchbox.common.db import sql_to_df
 from matchbox.common.dtos import (
-    ModelAncestor,
-    ModelConfig,
-    ModelType,
+    Collection,
+    CollectionName,
+    Match,
+    ModelResolutionPath,
+    Resolution,
+    ResolutionPath,
+    ResolutionType,
+    Run,
+    RunID,
+    SourceResolutionPath,
 )
 from matchbox.common.eval import Judgement as CommonJudgement
 from matchbox.common.eval import ModelComparison
 from matchbox.common.exceptions import (
+    MatchboxCollectionAlreadyExists,
     MatchboxDataNotFound,
     MatchboxDeletionNotConfirmed,
-    MatchboxModelConfigError,
     MatchboxNoJudgements,
-    MatchboxResolutionNotFoundError,
+    MatchboxRunNotWriteable,
 )
-from matchbox.common.graph import (
-    ModelResolutionName,
-    ResolutionGraph,
-    ResolutionName,
-    ResolutionNodeType,
-    SourceResolutionName,
-)
-from matchbox.common.sources import Match, SourceConfig
+from matchbox.common.logging import logger
 from matchbox.server.base import MatchboxDBAdapter, MatchboxSnapshot
 from matchbox.server.postgresql.db import (
     MBDB,
@@ -39,13 +40,14 @@ from matchbox.server.postgresql.db import (
 from matchbox.server.postgresql.orm import (
     Clusters,
     ClusterSourceKey,
+    Collections,
     Contains,
     EvalJudgements,
     PKSpace,
     Probabilities,
-    ResolutionFrom,
     Resolutions,
     Results,
+    Runs,
     SourceConfigs,
     Users,
 )
@@ -53,20 +55,13 @@ from matchbox.server.postgresql.utils import evaluation
 from matchbox.server.postgresql.utils.db import (
     compile_sql,
     dump,
-    get_resolution_graph,
     restore,
 )
 from matchbox.server.postgresql.utils.insert import (
-    insert_model,
+    insert_hashes,
     insert_results,
-    insert_source,
 )
-from matchbox.server.postgresql.utils.query import (
-    get_source_config,
-    match,
-    query,
-)
-from matchbox.server.postgresql.utils.results import get_model_config
+from matchbox.server.postgresql.utils.query import match, query
 
 T = TypeVar("T")
 P = ParamSpec("P")
@@ -145,7 +140,6 @@ class FilteredResolutions(BaseModel):
     """Wrapper class for filtered resolution queries."""
 
     sources: bool = False
-    humans: bool = False
     models: bool = False
 
     def count(self) -> int:
@@ -155,11 +149,9 @@ class FilteredResolutions(BaseModel):
 
             filter_list = []
             if self.sources:
-                filter_list.append(Resolutions.type == ResolutionNodeType.SOURCE)
-            if self.humans:
-                filter_list.append(Resolutions.type == ResolutionNodeType.HUMAN)
+                filter_list.append(Resolutions.type == ResolutionType.SOURCE)
             if self.models:
-                filter_list.append(Resolutions.type == ResolutionNodeType.MODEL)
+                filter_list.append(Resolutions.type == ResolutionType.MODEL)
 
             if filter_list:
                 query = query.filter(or_(*filter_list))
@@ -179,29 +171,27 @@ class MatchboxPostgres(MatchboxDBAdapter):
         PKSpace.initialise()
 
         self.sources = SourceConfigs
-        self.models = FilteredResolutions(sources=False, humans=False, models=True)
-        self.source_resolutions = FilteredResolutions(
-            sources=True, humans=False, models=False
-        )
+        self.models = FilteredResolutions(sources=False, models=True)
         self.data = FilteredClusters(has_source=True)
         self.clusters = FilteredClusters(has_source=False)
-        self.merges = Contains
         self.creates = FilteredProbabilities(over_truth=True)
+        self.merges = Contains
         self.proposes = FilteredProbabilities()
+        self.source_resolutions = FilteredResolutions(sources=True, models=False)
 
     # Retrieval
 
     def query(  # noqa: D102
         self,
-        source: SourceResolutionName,
-        resolution: ResolutionName | None = None,
+        source: SourceResolutionPath,
+        point_of_truth: ResolutionPath | None = None,
         threshold: int | None = None,
         return_leaf_id: bool = False,
         limit: int | None = None,
     ) -> ArrowTable:
         return query(
             source=source,
-            resolution=resolution,
+            point_of_truth=point_of_truth,
             threshold=threshold,
             return_leaf_id=return_leaf_id,
             limit=limit,
@@ -210,71 +200,228 @@ class MatchboxPostgres(MatchboxDBAdapter):
     def match(  # noqa: D102
         self,
         key: str,
-        source: SourceResolutionName,
-        targets: list[SourceResolutionName],
-        resolution: ResolutionName,
+        source: SourceResolutionPath,
+        targets: list[SourceResolutionPath],
+        point_of_truth: ResolutionPath,
         threshold: int | None = None,
     ) -> list[Match]:
         return match(
             key=key,
             source=source,
             targets=targets,
-            resolution=resolution,
+            point_of_truth=point_of_truth,
             threshold=threshold,
         )
 
-    # Data management
+    # Collection management
 
-    def index(self, source_config: SourceConfig, data_hashes: Table) -> None:  # noqa: D102
-        insert_source(
-            source_config=source_config,
-            data_hashes=data_hashes,
-            batch_size=self.settings.batch_size,
-        )
-
-    def get_source_config(self, name: SourceResolutionName) -> SourceConfig:  # noqa: D102
+    def create_collection(self, name: CollectionName) -> Collection:  # noqa: D102
         with MBDB.get_session() as session:
-            if source := get_source_config(name, session):
-                return source.to_dto()
+            if (session.query(Collections).filter_by(name=name).first()) is None:
+                new_collection = Collections(name=name)
+                session.add(new_collection)
+                session.commit()
+                return new_collection.to_dto()
+            else:
+                raise MatchboxCollectionAlreadyExists
 
-    def get_resolution_source_configs(  # noqa: D102
-        self,
-        name: ModelResolutionName,
-    ) -> list[SourceConfig]:
+    def get_collection(  # noqa: D102
+        self, name: CollectionName
+    ) -> Collection:
         with MBDB.get_session() as session:
-            # Find resolution by name
-            resolution: Resolutions | None = (
-                session.query(Resolutions).filter(Resolutions.name == name).first()
+            collection_orm = Collections.from_name(name, session)
+            return collection_orm.to_dto()
+
+    def list_collections(self) -> list[CollectionName]:  # noqa: D102
+        with MBDB.get_session() as session:
+            collections = (
+                session.execute(select(Collections.name).order_by(Collections.name))
+                .scalars()
+                .all()
             )
-            if not resolution:
-                raise MatchboxResolutionNotFoundError(name=name)
-            # Find all resolutions in scope (selected + ancestors)
-            relevant_resolutions = (
-                session.query(Resolutions)
-                .filter(
+            return list(collections)
+
+    def delete_collection(self, name: CollectionName, certain: bool) -> None:  # noqa: D102
+        with MBDB.get_session() as session:
+            collection_orm = Collections.from_name(name, session)
+
+            if not certain:
+                version_names = [v.name for v in collection_orm.versions]
+                raise MatchboxDeletionNotConfirmed(children=version_names)
+
+            session.execute(
+                delete(Collections).where(
+                    Collections.collection_id == collection_orm.collection_id
+                )
+            )
+            session.commit()
+
+    # Run management
+
+    def create_run(self, collection: CollectionName) -> Run:  # noqa: D102
+        with MBDB.get_session() as session:
+            # Can raise MatchboxCollectionNotFoundError
+            collection_orm = Collections.from_name(collection, session)
+
+            new_run = Runs(
+                collection_id=collection_orm.collection_id,
+                is_mutable=True,
+                is_default=False,
+            )
+            session.add(new_run)
+            session.commit()
+
+            return new_run.to_dto()
+
+    def set_run_mutable(  # noqa: D102
+        self, collection: CollectionName, run_id: RunID, mutable: bool
+    ) -> Run:
+        with MBDB.get_session() as session:
+            run_orm = Runs.from_id(collection, run_id, session)
+            run_orm.is_mutable = mutable
+            session.commit()
+
+            return run_orm.to_dto()
+
+    def set_run_default(  # noqa: D102
+        self, collection: CollectionName, run_id: RunID, default: bool
+    ) -> Run:
+        with MBDB.get_session() as session:
+            run_orm = Runs.from_id(collection, run_id, session)
+            if default:
+                if run_orm.is_mutable:
+                    raise ValueError("Cannot set as default a mutable run")
+                # Unset any existing default run for the collection
+                session.execute(
+                    update(Runs)
+                    .where(
+                        Runs.collection_id == run_orm.collection_id,
+                        Runs.is_default.is_(True),
+                    )
+                    .values(is_default=False)
+                )
+
+            run_orm.is_default = default
+            session.commit()
+
+            return run_orm.to_dto()
+
+    def get_run(self, collection: CollectionName, run_id: RunID) -> Run:  # noqa: D102
+        with MBDB.get_session() as session:
+            run_orm = Runs.from_id(collection, run_id, session)
+            return run_orm.to_dto()
+
+    def delete_run(  # noqa: D102
+        self, collection: CollectionName, run_id: RunID, certain: bool
+    ) -> None:
+        with MBDB.get_session() as session:
+            run_orm = Runs.from_id(collection, run_id, session)
+
+            if not certain:
+                resolution_names = [res.name for res in run_orm.resolutions]
+                raise MatchboxDeletionNotConfirmed(children=resolution_names)
+
+            session.execute(delete(Runs).where(Runs.run_id == run_orm.run_id))
+            session.commit()
+
+    # Resolution management
+
+    def _check_writeable(self, path: ResolutionPath):
+        run = Runs.from_id(collection=path.collection, run_id=path.run)
+        if not run.is_mutable:
+            raise MatchboxRunNotWriteable(
+                f"Version {path.run} in collection {path.collection} is immutable"
+            )
+
+    def create_resolution(  # noqa: D102
+        self, resolution: Resolution, path: ResolutionPath
+    ) -> None:
+        self._check_writeable(path)
+        log_prefix = f"Insert {path.name}"
+        with MBDB.get_session() as session:
+            resolution_orm = Resolutions.from_dto(
+                resolution=resolution, path=path, session=session
+            )
+            session.commit()
+
+            logger.info(
+                f"Inserted with ID {resolution_orm.resolution_id}", prefix=log_prefix
+            )
+
+    def get_resolution(  # noqa: D102
+        self, path: ResolutionPath, validate: ResolutionType | None = None
+    ) -> Resolution:
+        with MBDB.get_session() as session:
+            resolution = Resolutions.from_path(
+                path=path, res_type=validate, session=session
+            )
+            return resolution.to_dto()
+
+    def delete_resolution(self, path: ResolutionPath, certain: bool) -> None:  # noqa: D102
+        self._check_writeable(path)
+        with MBDB.get_session() as session:
+            resolution = Resolutions.from_path(path=path, session=session)
+            if certain:
+                delete_stmt = delete(Resolutions).where(
                     Resolutions.resolution_id.in_(
                         [
-                            res.resolution_id
-                            for res in resolution.ancestors.union([resolution])
+                            resolution.resolution_id,
+                            *(d.resolution_id for d in resolution.descendants),
                         ]
                     )
                 )
-                .subquery()
+                session.execute(delete_stmt)
+                session.commit()
+            else:
+                children = [r.name for r in resolution.descendants]
+                raise MatchboxDeletionNotConfirmed(children=children)
+
+    # Data insertion
+
+    def insert_source_data(  # noqa: D102
+        self, path: SourceResolutionPath, data_hashes: Table
+    ) -> None:
+        self._check_writeable(path)
+        insert_hashes(
+            path=path, data_hashes=data_hashes, batch_size=self.settings.batch_size
+        )
+
+    def insert_model_data(self, path: ModelResolutionPath, results: Table) -> None:  # noqa: D102
+        self._check_writeable(path)
+        insert_results(path=path, results=results, batch_size=self.settings.batch_size)
+
+    def get_model_data(self, path: ModelResolutionPath) -> Table:  # noqa: D102
+        with MBDB.get_session() as session:
+            resolution = Resolutions.from_path(
+                path=path, res_type=ResolutionType.MODEL, session=session
             )
 
-            # Find all sources matching a resolution in scope
-            res_sources: list[SourceConfigs] = (
-                session.query(SourceConfigs)
-                .join(
-                    relevant_resolutions,
-                    SourceConfigs.resolution_id == relevant_resolutions.c.resolution_id,
-                )
-                .all()
+            results_query = select(
+                Results.left_id, Results.right_id, Results.probability
+            ).where(Results.resolution_id == resolution.resolution_id)
+
+        with MBDB.get_adbc_connection() as conn:
+            stmt: str = compile_sql(results_query)
+            return sql_to_df(
+                stmt=stmt, connection=conn.dbapi_connection, return_type="arrow"
             )
 
-            return [s.to_dto() for s in res_sources]
+    def set_model_truth(self, path: ModelResolutionPath, truth: int) -> None:  # noqa: D102
+        with MBDB.get_session() as session:
+            self._check_writeable(path)
+            resolution = Resolutions.from_path(
+                path=path, res_type=ResolutionType.MODEL, session=session
+            )
+            resolution.truth = truth
+            session.commit()
 
-    def validate_ids(self, ids: list[int]) -> None:  # noqa: D102
+    def get_model_truth(self, path: ModelResolutionPath) -> int:  # noqa: D102
+        resolution = Resolutions.from_path(path=path, res_type=ResolutionType.MODEL)
+        return resolution.truth
+
+    # Data management
+
+    def validate_ids(self, ids: list[int]) -> bool:  # noqa: D102
         with MBDB.get_session() as session:
             data_inner_join = (
                 session.query(Clusters)
@@ -300,61 +447,12 @@ class MatchboxPostgres(MatchboxDBAdapter):
                 data=missing_ids,
             )
 
-    def validate_hashes(self, hashes: list[bytes]) -> None:  # noqa: D102
-        with MBDB.get_session() as session:
-            data_inner_join = (
-                session.query(Clusters)
-                .filter(
-                    Clusters.cluster_hash.in_(
-                        bindparam(
-                            "ins_hashs",
-                            hashes,
-                            expanding=True,
-                        )
-                    )
-                )
-                .all()
-            )
-
-        existing_hashes = {item.cluster_hash for item in data_inner_join}
-        missing_hashes = set(hashes) - existing_hashes
-
-        if missing_hashes:
-            raise MatchboxDataNotFound(
-                message="Some items don't exist in Clusters table.",
-                table=Clusters.__tablename__,
-                data=missing_hashes,
-            )
-
-    def cluster_id_to_hash(self, ids: list[int]) -> dict[int, bytes | None]:  # noqa: D102
-        initial_dict = {id: None for id in ids}
-
-        with MBDB.get_session() as session:
-            data_inner_join = (
-                session.query(Clusters)
-                .filter(
-                    Clusters.cluster_id.in_(
-                        bindparam(
-                            "ins_ids",
-                            ids,
-                            expanding=True,
-                        )
-                    )
-                )
-                .all()
-            )
-
-        return initial_dict | {
-            item.cluster_id: item.cluster_hash for item in data_inner_join
-        }
-
-    def get_resolution_graph(self) -> ResolutionGraph:  # noqa: D102
-        return get_resolution_graph()
+        return True
 
     def dump(self) -> MatchboxSnapshot:  # noqa: D102
         return dump()
 
-    def drop(self, certain: bool = False) -> None:  # noqa: D102
+    def drop(self, certain: bool) -> None:  # noqa: D102
         if certain:
             MBDB.drop_database()
             PKSpace.initialise()
@@ -365,7 +463,7 @@ class MatchboxPostgres(MatchboxDBAdapter):
                 "If you're sure you want to continue, rerun with certain=True"
             )
 
-    def clear(self, certain: bool = False) -> None:  # noqa: D102
+    def clear(self, certain: bool) -> None:  # noqa: D102
         if certain:
             MBDB.clear_database()
             PKSpace.initialise()
@@ -389,160 +487,7 @@ class MatchboxPostgres(MatchboxDBAdapter):
             batch_size=self.settings.batch_size,
         )
 
-    # Model management
-
-    def insert_model(self, model_config: ModelConfig) -> None:  # noqa: D102
-        with MBDB.get_session() as session:
-            left_resolution = (
-                session.query(Resolutions)
-                .filter(Resolutions.name == model_config.left_resolution)
-                .first()
-            )
-            if not left_resolution:
-                raise MatchboxResolutionNotFoundError(name=model_config.left_resolution)
-
-            # Overwritten with actual right model if in a link job
-            right_resolution = left_resolution
-            if model_config.type == ModelType.LINKER:
-                right_resolution = (
-                    session.query(Resolutions)
-                    .filter(Resolutions.name == model_config.right_resolution)
-                    .first()
-                )
-                if not right_resolution:
-                    raise MatchboxResolutionNotFoundError(
-                        name=model_config.right_resolution
-                    )
-
-                left_ancestors = {a.name for a in left_resolution.ancestors}
-                right_ancestors = {a.name for a in right_resolution.ancestors}
-                shared_ancestors = left_ancestors & right_ancestors
-
-                if shared_ancestors:
-                    raise MatchboxModelConfigError(
-                        f"Resolutions '{left_resolution.name}' and "
-                        f"'{right_resolution.name}' "
-                        f"share common ancestor(s): {', '.join(shared_ancestors)}. "
-                        f"Resolutions cannot share ancestors."
-                    )
-
-        insert_model(
-            name=model_config.name,
-            left=left_resolution,
-            right=right_resolution,
-            description=model_config.description,
-        )
-
-    def get_model(self, name: ModelResolutionName) -> ModelConfig:  # noqa: D102
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        return get_model_config(resolution=resolution)
-
-    def set_model_results(self, name: ModelResolutionName, results: Table) -> None:  # noqa: D102
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        insert_results(
-            results=results,
-            resolution=resolution,
-            batch_size=self.settings.batch_size,
-        )
-
-    def get_model_results(self, name: ModelResolutionName) -> Table:  # noqa: D102
-        results_query = (
-            select(Results.left_id, Results.right_id, Results.probability)
-            .join(
-                Resolutions,
-                Results.resolution_id == Resolutions.resolution_id,
-            )
-            .where(
-                Resolutions.name == name, Resolutions.type == ResolutionNodeType.MODEL
-            )
-        )
-        with MBDB.get_adbc_connection() as conn:
-            stmt: str = compile_sql(results_query)
-            return sql_to_df(
-                stmt=stmt, connection=conn.dbapi_connection, return_type="arrow"
-            )
-
-    def set_model_truth(self, name: ModelResolutionName, truth: int) -> None:  # noqa: D102
-        with MBDB.get_session() as session:
-            resolution = Resolutions.from_name(
-                name=name, res_type="model", session=session
-            )
-            resolution.truth = truth
-            session.commit()
-
-    def get_model_truth(self, name: ModelResolutionName) -> int:  # noqa: D102
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        return resolution.truth
-
-    def get_model_ancestors(self, name: ModelResolutionName) -> list[ModelAncestor]:  # noqa: D102
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        return [
-            ModelAncestor(name=resolution.name, truth=resolution.truth)
-            for resolution in resolution.ancestors
-        ]
-
-    def set_model_ancestors_cache(  # noqa: D102
-        self,
-        name: ModelResolutionName,
-        ancestors_cache: list[ModelAncestor],
-    ) -> None:
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        with MBDB.get_session() as session:
-            ancestor_names = [ancestor.name for ancestor in ancestors_cache]
-            name_to_id = dict(
-                session.query(Resolutions.name, Resolutions.resolution_id)
-                .filter(Resolutions.name.in_(ancestor_names))
-                .all()
-            )
-
-            for ancestor in ancestors_cache:
-                parent_id = name_to_id.get(ancestor.name)
-                if parent_id is None:
-                    raise ValueError(f"Model '{ancestor.name}' not found in database")
-
-                session.execute(
-                    ResolutionFrom.__table__.update()
-                    .where(ResolutionFrom.parent == parent_id)
-                    .where(ResolutionFrom.child == resolution.resolution_id)
-                    .values(truth_cache=ancestor.truth)
-                )
-
-            session.commit()
-
-    def get_model_ancestors_cache(  # noqa: D102
-        self, name: ModelResolutionName
-    ) -> list[ModelAncestor]:
-        resolution = Resolutions.from_name(name=name, res_type="model")
-        with MBDB.get_session() as session:
-            query = (
-                select(Resolutions.name, ResolutionFrom.truth_cache)
-                .join(Resolutions, Resolutions.resolution_id == ResolutionFrom.parent)
-                .where(ResolutionFrom.child == resolution.resolution_id)
-                .where(ResolutionFrom.truth_cache.isnot(None))
-            )
-
-            return [
-                ModelAncestor(name=name, truth=truth)
-                for name, truth in session.execute(query).all()
-            ]
-
-    def delete_resolution(self, name: ResolutionName, certain: bool = False) -> None:  # noqa: D102
-        resolution = Resolutions.from_name(name=name)
-        with MBDB.get_session() as session:
-            if certain:
-                delete_stmt = delete(Resolutions).where(
-                    Resolutions.resolution_id.in_(
-                        [
-                            resolution.resolution_id,
-                            *(d.resolution_id for d in resolution.descendants),
-                        ]
-                    )
-                )
-                session.execute(delete_stmt)
-                session.commit()
-            else:
-                children = [r.name for r in resolution.descendants]
-                raise MatchboxDeletionNotConfirmed(childen=children)
+    # User management
 
     def login(self, user_name: str) -> int:  # noqa: D102
         with MBDB.get_session() as session:
@@ -557,6 +502,8 @@ class MatchboxPostgres(MatchboxDBAdapter):
 
             return user.user_id
 
+    # Evaluation management
+
     def insert_judgement(self, judgement: CommonJudgement) -> None:  # noqa: D102
         # Check that all referenced cluster IDs exist
         ids = list(chain(*judgement.endorsed)) + [judgement.shown]
@@ -566,13 +513,15 @@ class MatchboxPostgres(MatchboxDBAdapter):
     def get_judgements(self) -> tuple[Table, Table]:  # noqa: D102
         return evaluation.get_judgements()
 
-    def compare_models(self, resolutions: list[ModelResolutionName]) -> ModelComparison:  # noqa: D102
+    def compare_models(self, paths: list[ModelResolutionPath]) -> ModelComparison:  # noqa: D102
         judgements, expansion = self.get_judgements()
         if not len(judgements):
             raise MatchboxNoJudgements()
-        return evaluation.compare_models(resolutions, judgements, expansion)
+        return evaluation.compare_models(
+            paths, pl.from_arrow(judgements), pl.from_arrow(expansion)
+        )
 
     def sample_for_eval(  # noqa: D102
-        self, n: int, resolution: ModelResolutionName, user_id: int
+        self, n: int, path: ModelResolutionPath, user_id: int
     ) -> ArrowTable:
-        return evaluation.sample(n, resolution, user_id)
+        return evaluation.sample(n, path, user_id)
