@@ -1,5 +1,6 @@
 """ORM classes for the Matchbox PostgreSQL database."""
 
+import json
 from typing import Literal
 
 from sqlalchemy import (
@@ -18,19 +19,180 @@ from sqlalchemy import (
     text,
     update,
 )
-from sqlalchemy.dialects.postgresql import BYTEA, TEXT, insert
-from sqlalchemy.orm import Session, relationship
+from sqlalchemy.dialects.postgresql import BYTEA, JSONB, TEXT, insert
+from sqlalchemy.orm import Session, relationship, selectinload
 
+from matchbox.common.dtos import Collection as CommonCollection
+from matchbox.common.dtos import (
+    CollectionName,
+    LocationConfig,
+    ModelType,
+    ResolutionName,
+    ResolutionPath,
+    ResolutionType,
+    RunID,
+)
+from matchbox.common.dtos import ModelConfig as CommonModelConfig
+from matchbox.common.dtos import Resolution as CommonResolution
+from matchbox.common.dtos import Run as CommonRun
+from matchbox.common.dtos import SourceConfig as CommonSourceConfig
+from matchbox.common.dtos import SourceField as CommonSourceField
 from matchbox.common.exceptions import (
+    MatchboxCollectionNotFoundError,
+    MatchboxResolutionAlreadyExists,
     MatchboxResolutionNotFoundError,
+    MatchboxRunNotFoundError,
 )
-from matchbox.common.graph import ResolutionName
-from matchbox.common.sources import SourceConfig as CommonSourceConfig
-from matchbox.common.sources import SourceField as CommonSourceField
 from matchbox.server.postgresql.db import MBDB
 from matchbox.server.postgresql.mixin import CountMixin
 
 
+class Collections(CountMixin, MBDB.MatchboxBase):
+    """Named collections of resolutions and runs."""
+
+    __tablename__ = "collections"
+
+    collection_id = Column(BIGINT, primary_key=True, autoincrement=True)
+    name = Column(TEXT, nullable=False)
+
+    # Relationships
+    runs = relationship("Runs", back_populates="collection")
+
+    # Constraints
+    __table_args__ = (UniqueConstraint("name", name="collections_name_key"),)
+
+    @classmethod
+    def from_name(
+        cls,
+        name: CollectionName,
+        session: Session | None = None,
+    ) -> "Collections":
+        """Resolve a collection name to a Collections object.
+
+        Args:
+            name: The name of the collection to resolve.
+            session: Optional session to use for the query.
+
+        Raises:
+            MatchboxCollectionNotFoundError: If the collection doesn't exist.
+        """
+        query = select(cls).where(cls.name == name).options(selectinload(cls.runs))
+
+        if session:
+            collection = session.execute(query).scalar_one_or_none()
+        else:
+            with MBDB.get_session() as session:
+                collection = session.execute(query).scalar_one_or_none()
+
+        if not collection:
+            raise MatchboxCollectionNotFoundError(f"Collection '{name}' not found.")
+
+        return collection
+
+    def to_dto(self) -> CommonCollection:
+        """Convert ORM collection to a matchbox.common Collection object."""
+        run_ids: list[RunID] = []
+        default_run = None
+        if runs := self.runs:
+            run_ids = [r.run_id for r in runs]
+            default_run_list = [r.run_id for r in runs if r.is_default]
+            if default_run_list:
+                default_run = default_run_list[0]
+
+        return CommonCollection(runs=run_ids, default_run=default_run)
+
+
+class Runs(CountMixin, MBDB.MatchboxBase):
+    """Runs of collections of resolutions."""
+
+    __tablename__ = "runs"
+
+    run_id = Column(BIGINT, primary_key=True, autoincrement=True)
+    collection_id = Column(
+        BIGINT,
+        ForeignKey("collections.collection_id", ondelete="CASCADE"),
+        nullable=False,
+    )
+    is_mutable = Column(BOOLEAN, default=False)
+    is_default = Column(BOOLEAN, default=False)
+
+    # Relationships
+    collection = relationship("Collections", back_populates="runs")
+    resolutions = relationship("Resolutions", back_populates="run")
+
+    # Constraints
+    __table_args__ = (
+        UniqueConstraint("collection_id", "run_id", name="unique_run_id"),
+        Index(
+            "ix_default_run_collection",
+            "collection_id",
+            unique=True,
+            postgresql_where=text("is_default = true"),
+        ),
+    )
+
+    @classmethod
+    def from_id(
+        cls,
+        collection: CollectionName,
+        run_id: RunID,
+        session: Session | None = None,
+    ) -> "Runs":
+        """Resolve a collection and run name to a Runs object.
+
+        Args:
+            collection: The name of the collection containing the run.
+            run_id: The ID of the run within that collection.
+            session: Optional session to use for the query.
+
+        Raises:
+            MatchboxRunNotFoundError: If the run doesn't exist.
+        """
+        query = (
+            select(cls)
+            .where(cls.run_id == run_id)
+            .options(
+                selectinload(cls.resolutions)
+                .selectinload(Resolutions.source_config)
+                .selectinload(SourceConfigs.fields),
+                selectinload(cls.resolutions).selectinload(Resolutions.model_config),
+                selectinload(cls.collection),
+            )
+        )
+
+        if session:
+            run_orm = session.execute(query).scalar_one_or_none()
+        else:
+            with MBDB.get_session() as session:
+                run_orm = session.execute(query).scalar_one_or_none()
+
+        if not run_orm:
+            raise MatchboxRunNotFoundError
+
+        if run_orm.collection.name != collection:
+            raise MatchboxRunNotFoundError(
+                run_id=id,
+                message=f"Run {id} not found in collection {collection}",
+            )
+
+        return run_orm
+
+    def to_dto(self) -> CommonRun:
+        """Convert ORM run to a matchbox.common Run object."""
+        resolutions: dict[ResolutionName, CommonResolution] = {}
+        if self.resolutions:
+            resolutions = {
+                resolution.name: resolution.to_dto() for resolution in self.resolutions
+            }
+
+        return CommonRun(
+            run_id=self.run_id,
+            is_default=self.is_default,
+            is_mutable=self.is_mutable,
+            resolutions=resolutions,
+        )
+
+
 class ResolutionFrom(CountMixin, MBDB.MatchboxBase):
     """Resolution lineage closure table with cached truth values."""
 
@@ -58,7 +220,7 @@ class ResolutionFrom(CountMixin, MBDB.MatchboxBase):
 
 
 class Resolutions(CountMixin, MBDB.MatchboxBase):
-    """Table of resolution points: models, sources and humans.
+    """Table of resolution points corresponding to models, and sources.
 
     Resolutions produce probabilities or own data in the clusters table.
     """
@@ -67,6 +229,9 @@ class Resolutions(CountMixin, MBDB.MatchboxBase):
 
     # Columns
     resolution_id = Column(BIGINT, primary_key=True, autoincrement=True)
+    run_id = Column(
+        BIGINT, ForeignKey("runs.run_id", ondelete="CASCADE"), nullable=False
+    )
     name = Column(TEXT, nullable=False)
     description = Column(TEXT, nullable=True)
     type = Column(TEXT, nullable=False)
@@ -77,6 +242,9 @@ class Resolutions(CountMixin, MBDB.MatchboxBase):
     source_config = relationship(
         "SourceConfigs", back_populates="source_resolution", uselist=False
     )
+    model_config = relationship(
+        "ModelConfigs", back_populates="model_resolution", uselist=False
+    )
     probabilities = relationship(
         "Probabilities",
         back_populates="proposed_by",
@@ -94,14 +262,15 @@ class Resolutions(CountMixin, MBDB.MatchboxBase):
         secondaryjoin="Resolutions.resolution_id == ResolutionFrom.child",
         backref="parents",
     )
+    run = relationship("Runs", back_populates="resolutions")
 
     # Constraints
     __table_args__ = (
         CheckConstraint(
-            "type IN ('model', 'source', 'human')",
+            "type IN ('model', 'source')",
             name="resolution_type_constraints",
         ),
-        UniqueConstraint("name", name="resolutions_name_key"),
+        UniqueConstraint("run_id", "name", name="resolutions_name_key"),
     )
 
     @property
@@ -196,25 +365,35 @@ class Resolutions(CountMixin, MBDB.MatchboxBase):
             )
 
     @classmethod
-    def from_name(
+    def from_path(
         cls,
-        name: ResolutionName,
-        res_type: Literal["model", "source", "human"] | None = None,
+        path: ResolutionPath,
+        res_type: ResolutionType | None = None,
         session: Session | None = None,
     ) -> "Resolutions":
-        """Resolves a model resolution name to a Resolution object.
+        """Resolves a resolution name to a Resolution object.
 
         Args:
-            name: The name of the model to resolve.
+            path: The path of the resolution to resolve.
             res_type: A resolution type to use as filter.
             session: A session to get the resolution for updates.
 
         Raises:
-            MatchboxResolutionNotFoundError: If the model doesn't exist.
+            MatchboxResolutionNotFoundError: If the resolution doesn't exist.
         """
-        query = select(cls).where(cls.name == name)
+        query = (
+            select(cls)
+            .join(cls.run)
+            .join(Runs.collection)
+            .where(
+                cls.name == path.name,
+                Runs.run_id == path.run,
+                Collections.name == path.collection,
+            )
+        )
+
         if res_type:
-            query = query.where(cls.type == res_type)
+            query = query.where(cls.type == res_type.value)
 
         if session:
             resolution = session.execute(query).scalar()
@@ -225,11 +404,122 @@ class Resolutions(CountMixin, MBDB.MatchboxBase):
         if resolution:
             return resolution
 
-        res_type = res_type or "any"
         raise MatchboxResolutionNotFoundError(
-            message=f"No resolution {name} of {res_type}."
+            message=f"No resolution {path} of type {res_type or 'any'}."
         )
 
+    @classmethod
+    def from_dto(
+        cls, resolution: CommonResolution, path: ResolutionPath, session: Session
+    ) -> "Resolutions":
+        """Create a Resolutions instance from a Resolution DTO object.
+
+        The resolution will be added to the session and flushed (but not committed).
+
+        For model resolutions, lineage entries will be created automatically.
+
+        Args:
+            resolution: The Resolution DTO to convert
+            path: The full resolution path
+            session: Database session (caller must commit)
+
+        Returns:
+            A Resolutions ORM instance with ID and relationships established
+        """
+        # Find the run ID for the given collection and run ID
+        run_obj = session.execute(
+            select(Runs)
+            .join(Collections)
+            .where(Collections.name == path.collection, Runs.run_id == path.run)
+        ).scalar_one_or_none()
+
+        if not run_obj:
+            raise MatchboxRunNotFoundError(number=path.run)
+
+        # Check if resolution already exists within run
+        existing_resolutions: Resolutions = run_obj.resolutions
+        for res in existing_resolutions:
+            if res.name == path.name:
+                raise MatchboxResolutionAlreadyExists(
+                    f"Resolution {path.name} already exists"
+                )
+
+        # Create new resolution
+        resolution_orm = cls(
+            run_id=run_obj.run_id,
+            name=path.name,
+            description=resolution.description,
+            type=resolution.resolution_type.value,
+            truth=resolution.truth,
+        )
+        session.add(resolution_orm)
+        session.flush()  # Get resolution_id
+
+        if resolution.resolution_type == ResolutionType.SOURCE:
+            resolution_orm.source_config = SourceConfigs.from_dto(resolution.config)
+
+        elif resolution.resolution_type == ResolutionType.MODEL:
+            resolution_orm.model_config = ModelConfigs.from_dto(resolution.config)
+            # Create lineage
+            left_parent = cls.from_path(
+                resolution.config.left_query.point_of_truth, session=session
+            )
+            cls._create_closure_entries(session, resolution_orm, left_parent)
+
+            if resolution.config.type == ModelType.LINKER:
+                right_parent = cls.from_path(
+                    resolution.config.right_query.point_of_truth, session=session
+                )
+                cls._create_closure_entries(session, resolution_orm, right_parent)
+
+        return resolution_orm
+
+    def to_dto(self) -> CommonResolution:
+        """Convert ORM resolution to a matchbox.common Resolution object."""
+        if self.type == ResolutionType.SOURCE:
+            config = self.source_config.to_dto()
+        else:
+            config = self.model_config.to_dto()
+
+        return CommonResolution(
+            description=self.description,
+            truth=self.truth,
+            resolution_type=ResolutionType(self.type),
+            config=config,
+        )
+
+    @staticmethod
+    def _create_closure_entries(
+        session: Session, child: "Resolutions", parent: "Resolutions"
+    ):
+        """Create closure table entries for a parent-child relationship."""
+        # Direct relationship
+        session.add(
+            ResolutionFrom(
+                parent=parent.resolution_id,
+                child=child.resolution_id,
+                level=1,
+                truth_cache=parent.truth,
+            )
+        )
+
+        # Transitive closure
+        ancestors = (
+            session.query(ResolutionFrom)
+            .filter(ResolutionFrom.child == parent.resolution_id)
+            .all()
+        )
+
+        for ancestor in ancestors:
+            session.add(
+                ResolutionFrom(
+                    parent=ancestor.parent,
+                    child=child.resolution_id,
+                    level=ancestor.level + 1,
+                    truth_cache=ancestor.truth_cache,
+                )
+            )
+
 
 class PKSpace(MBDB.MatchboxBase):
     """Table used to reserve ranges of primary keys."""
@@ -443,19 +733,18 @@ class SourceConfigs(CountMixin, MBDB.MatchboxBase):
     @classmethod
     def from_dto(
         cls,
-        resolution: "Resolutions",
-        source_config: CommonSourceConfig,
+        config: CommonSourceConfig,
     ) -> "SourceConfigs":
-        """Create a SourceConfigs instance from a CommonSource object."""
+        """Create a SourceConfigs instance from a Resolution DTO object."""
+        # Create the SourceConfigs object
         return cls(
-            resolution_id=resolution.resolution_id,
-            location_type=source_config.location.type,
-            location_name=str(source_config.location.name),
-            extract_transform=source_config.extract_transform,
+            location_type=str(config.location_config.type),
+            location_name=str(config.location_config.name),
+            extract_transform=config.extract_transform,
             key_field=SourceFields(
                 index=0,
-                name=source_config.key_field.name,
-                type=source_config.key_field.type.value,
+                name=config.key_field.name,
+                type=config.key_field.type.value,
             ),
             index_fields=[
                 SourceFields(
@@ -463,18 +752,16 @@ class SourceConfigs(CountMixin, MBDB.MatchboxBase):
                     name=field.name,
                     type=field.type.value,
                 )
-                for idx, field in enumerate(source_config.index_fields)
+                for idx, field in enumerate(config.index_fields)
             ],
         )
 
     def to_dto(self) -> CommonSourceConfig:
-        """Convert ORM source to a matchbox.common SourceConfig object."""
+        """Convert ORM source to a matchbox.common.SourceConfig object."""
         return CommonSourceConfig(
-            name=self.name,
-            location={
-                "type": self.location_type,
-                "name": self.location_name,
-            },
+            location_config=LocationConfig(
+                type=self.location_type, name=self.location_name
+            ),
             extract_transform=self.extract_transform,
             key_field=CommonSourceField(
                 name=self.key_field.name, type=self.key_field.type
@@ -489,6 +776,64 @@ class SourceConfigs(CountMixin, MBDB.MatchboxBase):
         )
 
 
+class ModelConfigs(CountMixin, MBDB.MatchboxBase):
+    """Table of model configs for Matchbox."""
+
+    __tablename__ = "model_configs"
+
+    # Columns
+    model_config_id = Column(BIGINT, Identity(start=1), primary_key=True)
+    resolution_id = Column(
+        BIGINT,
+        ForeignKey("resolutions.resolution_id", ondelete="CASCADE"),
+        nullable=False,
+    )
+    model_class = Column(TEXT, nullable=False)
+    model_settings = Column(JSONB, nullable=False)
+    left_query = Column(JSONB, nullable=False)
+    right_query = Column(JSONB, nullable=True)
+
+    @property
+    def name(self) -> str:
+        """Get the name of the related resolution."""
+        return self.model_resolution.name
+
+    # Relationships
+    model_resolution = relationship("Resolutions", back_populates="model_config")
+
+    @classmethod
+    def list_all(cls) -> list["SourceConfigs"]:
+        """Returns all model_configs in the database."""
+        with MBDB.get_session() as session:
+            return session.query(cls).all()
+
+    @classmethod
+    def from_dto(
+        cls,
+        config: CommonModelConfig,
+    ) -> "ModelConfigs":
+        """Create a SourceConfigs instance from a Resolution DTO object."""
+        # Create the SourceConfigs object
+        return cls(
+            model_class=config.model_class,
+            model_settings=config.model_settings,
+            left_query=config.left_query.model_dump_json(),
+            right_query=(
+                None if not config.right_query else config.right_query.model_dump_json()
+            ),
+        )
+
+    def to_dto(self) -> CommonModelConfig:
+        """Convert ORM source to a matchbox.common.ModelConfig object."""
+        return CommonModelConfig(
+            type=ModelType.LINKER if self.right_query else ModelType.DEDUPER,
+            model_class=self.model_class,
+            model_settings=self.model_settings,
+            left_query=json.loads(self.left_query),
+            right_query=json.loads(self.right_query) if self.right_query else None,
+        )
+
+
 class Contains(CountMixin, MBDB.MatchboxBase):
     """Cluster lineage table."""
 
