@@ -14,17 +14,12 @@ from sqlalchemy import (
     select,
     union_all,
 )
-from sqlalchemy.orm import Session, aliased
+from sqlalchemy.orm import aliased
 from sqlalchemy.sql.selectable import Select
 
 from matchbox.common.db import sql_to_df
-from matchbox.common.exceptions import (
-    MatchboxResolutionNotFoundError,
-    MatchboxSourceNotFoundError,
-)
-from matchbox.common.graph import ResolutionName, SourceResolutionName
+from matchbox.common.dtos import Match, ResolutionPath, SourceResolutionPath
 from matchbox.common.logging import logger
-from matchbox.common.sources import Match
 from matchbox.server.postgresql.db import MBDB
 from matchbox.server.postgresql.orm import (
     Clusters,
@@ -40,20 +35,6 @@ from matchbox.server.postgresql.utils.db import compile_sql
 T = TypeVar("T")
 
 
-def get_source_config(name: SourceResolutionName, session: Session) -> SourceConfigs:
-    """Converts the named source to a SourceConfigs ORM object."""
-    source_config = (
-        session.query(SourceConfigs)
-        .join(Resolutions, Resolutions.resolution_id == SourceConfigs.resolution_id)
-        .filter(Resolutions.name == name)
-        .first()
-    )
-    if source_config is None:
-        raise MatchboxSourceNotFoundError(name=name)
-
-    return source_config
-
-
 def _build_probability_subquery(
     res_id: int, threshold_val: float, subquery_name: str
 ) -> Select:
@@ -467,8 +448,8 @@ def _build_match_query(
 
 
 def query(
-    source: SourceResolutionName,
-    resolution: ResolutionName | None = None,
+    source: SourceResolutionPath,
+    point_of_truth: ResolutionPath | None = None,
     threshold: int | None = None,
     return_leaf_id: bool = False,
     limit: int = None,
@@ -488,19 +469,15 @@ def query(
     Returns all records with their final resolved cluster IDs.
     """
     with MBDB.get_session() as session:
-        source_config: SourceConfigs = get_source_config(source, session)
-        source_resolution: Resolutions = session.get(
-            Resolutions, source_config.resolution_id
-        )
+        source_config: SourceConfigs = Resolutions.from_path(
+            path=source, session=session
+        ).source_config
+        source_resolution: Resolutions = source_config.source_resolution
 
-        if resolution:
-            truth_resolution: Resolutions = (
-                session.query(Resolutions)
-                .filter(Resolutions.name == resolution)
-                .first()
+        if point_of_truth:
+            truth_resolution: Resolutions = Resolutions.from_path(
+                path=point_of_truth, session=session
             )
-            if truth_resolution is None:
-                raise MatchboxResolutionNotFoundError(name=resolution)
         else:
             truth_resolution: Resolutions = source_resolution
 
@@ -589,9 +566,9 @@ def get_parent_clusters_and_leaves(
 
 def match(
     key: str,
-    source: SourceResolutionName,
-    targets: list[SourceResolutionName],
-    resolution: ResolutionName,
+    source: SourceResolutionPath,
+    targets: list[SourceResolutionPath],
+    point_of_truth: ResolutionPath,
     threshold: int | None = None,
 ) -> list[Match]:
     """Matches an ID in a source resolution and returns the keys in the targets.
@@ -611,16 +588,17 @@ def match(
     for each target.
     """
     with MBDB.get_session() as session:
-        # Get configurations
-        source_config: SourceConfigs = get_source_config(source, session)
-        truth_resolution: Resolutions | None = session.execute(
-            select(Resolutions).where(Resolutions.name == resolution)
-        ).scalar()
-        if truth_resolution is None:
-            raise MatchboxResolutionNotFoundError(name=resolution)
+        # Get configurations using ORM relationships
+        source_config: SourceConfigs = Resolutions.from_path(
+            path=source, session=session
+        ).source_config
+        truth_resolution: Resolutions = Resolutions.from_path(
+            path=point_of_truth, session=session
+        )
 
         target_configs: list[SourceConfigs] = [
-            get_source_config(target, session) for target in targets
+            Resolutions.from_path(path=target, session=session).source_config
+            for target in targets
         ]
         target_source_config_ids = [tc.source_config_id for tc in target_configs]
 
