@@ -1,6 +1,6 @@
 import logging
+from collections.abc import Iterator
 from pathlib import Path
-from typing import Iterator
 from unittest.mock import patch
 
 import pytest
@@ -9,7 +9,6 @@ from rich.progress import Progress
 
 pytest_plugins = [
     "test.fixtures.db",
-    "test.fixtures.graph",
     "test.fixtures.client",
 ]
 
