@@ -13,7 +13,11 @@ from matchbox.common.arrow import (
     SCHEMA_JUDGEMENTS,
     JudgementsZipFilenames,
 )
-from matchbox.common.dtos import BackendParameterType, BackendResourceType
+from matchbox.common.dtos import (
+    BackendParameterType,
+    BackendResourceType,
+    ModelResolutionPath,
+)
 from matchbox.common.eval import Judgement
 from matchbox.common.exceptions import (
     MatchboxDataNotFound,
@@ -95,41 +99,42 @@ def test_get_judgements(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
 
 
 def test_compare_models_ok(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
-    """Test errors in requesting samples."""
     test_client, mock_backend, _ = api_client_and_mocks
-    mock_pr = {"a": (1, 0.5), "b": (0.5, 1)}
-    mock_backend.compare_models = Mock(return_value=mock_pr)
+    model_a_path = ModelResolutionPath(name="a", collection="default", run=1)
+    model_b_path = ModelResolutionPath(name="b", collection="default", run=1)
+    mock_pr = {model_a_path: (1, 0.5), model_b_path: (0.5, 1)}
+    mock_backend.compare_models.return_value = mock_pr
 
-    response = test_client.get(
+    response = test_client.post(
         "/eval/compare",
-        params={"resolutions": ["a", "b"]},
+        json=[m.model_dump() for m in [model_a_path, model_b_path]],
     )
 
     assert response.status_code == 200
     result = response.json()
-    assert sorted(result.keys()) == ["a", "b"]
-    assert tuple(result["a"]) == mock_pr["a"]
-    assert tuple(result["b"]) == mock_pr["b"]
+    assert sorted(result.keys()) == ["default/1/a", "default/1/b"]
+    assert tuple(result["default/1/a"]) == mock_pr[model_a_path]
+    assert tuple(result["default/1/b"]) == mock_pr[model_b_path]
 
 
 def test_compare_models_404(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
-    """Test errors in requesting samples."""
     test_client, mock_backend, _ = api_client_and_mocks
+    model_a_path = ModelResolutionPath(name="a", collection="default", run=1)
+    model_b_path = ModelResolutionPath(name="b", collection="default", run=1)
+    model_c_path = ModelResolutionPath(name="c", collection="default", run=1)
 
-    # Resolution not found
-    mock_backend.compare_models = Mock(side_effect=MatchboxResolutionNotFoundError)
-    response = test_client.get(
+    mock_backend.compare_models.side_effect = MatchboxResolutionNotFoundError
+    response = test_client.post(
         "/eval/compare",
-        params={"resolutions": ["a", "b", "c"]},
+        json=[m.model_dump() for m in [model_a_path, model_b_path, model_c_path]],
     )
     assert response.status_code == 404
     assert response.json()["entity"] == BackendResourceType.RESOLUTION
 
-    # No judgements available
-    mock_backend.compare_models = Mock(side_effect=MatchboxNoJudgements)
-    response = test_client.get(
+    mock_backend.compare_models.side_effect = MatchboxNoJudgements
+    response = test_client.post(
         "/eval/compare",
-        params={"resolutions": ["a", "b", "c"]},
+        json=[m.model_dump() for m in [model_a_path, model_b_path, model_c_path]],
     )
     assert response.status_code == 404
     assert response.json()["entity"] == BackendResourceType.JUDGEMENT
@@ -153,13 +158,18 @@ def test_get_samples(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     # Process response
     response = test_client.get(
         "/eval/samples",
-        params={"n": 10, "resolution": "resolution", "user_id": 12},
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "resolution": "a",
+            "n": 10,
+            "user_id": 12,
+        },
     )
+    assert response.status_code == 200
+
     buffer = BytesIO(response.content)
     table = pq.read_table(buffer)
-
-    # Check response
-    assert response.status_code == 200
     assert table.equals(sample)
 
 
@@ -189,7 +199,13 @@ def test_get_samples_404(
 
     response = test_client.get(
         "/eval/samples",
-        params={"n": 10, "resolution": "resolution", "user_id": 12},
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "resolution": "a",
+            "n": 10,
+            "user_id": 12,
+        },
     )
 
     assert response.status_code == 404
@@ -203,7 +219,13 @@ def test_get_samples_422(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
 
     response = test_client.get(
         "/eval/samples",
-        params={"n": 10, "resolution": "resolution", "user_id": 12},
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "resolution": "a",
+            "n": 10,
+            "user_id": 12,
+        },
     )
 
     assert response.status_code == 422
