@@ -15,18 +15,19 @@ from matchbox.common.dtos import (
     BackendResourceType,
     LoginAttempt,
     LoginResult,
+    Match,
     OKMessage,
+    SourceResolutionPath,
     UploadStage,
     UploadStatus,
 )
 from matchbox.common.exceptions import (
+    MatchboxCollectionNotFoundError,
     MatchboxDeletionNotConfirmed,
     MatchboxResolutionNotFoundError,
-    MatchboxSourceNotFoundError,
+    MatchboxRunNotFoundError,
 )
 from matchbox.common.factories.sources import source_factory
-from matchbox.common.graph import ResolutionGraph
-from matchbox.common.sources import Match
 
 if TYPE_CHECKING:
     from mypy_boto3_s3.client import S3Client
@@ -75,7 +76,8 @@ def test_upload(
 
     mock_backend.settings.datastore.get_client.return_value = s3
     mock_backend.settings.datastore.cache_bucket_name = "test-bucket"
-    mock_backend.index = Mock(return_value=None)
+    mock_backend.create_resolution = Mock(return_value=None)
+    mock_backend.insert_source_data = Mock(return_value=None)
     s3.create_bucket(
         Bucket="test-bucket",
         CreateBucketConfiguration={"LocationConstraint": "eu-west-2"},
@@ -83,7 +85,7 @@ def test_upload(
 
     source_testkit = source_factory()
 
-    update_id = mock_tracker.add_source(source_testkit.source_config)
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
 
     # Make request with mocked background task
     response = test_client.post(
@@ -107,10 +109,93 @@ def test_upload(
     assert mock_tracker.update.call_args_list == [
         call(update_id, UploadStage.QUEUED),
     ]
-    mock_backend.index.assert_not_called()  # Index happens in background
+    mock_backend.create_resolution.assert_not_called()  # Index happens in background
+    mock_backend.insert_source_data.assert_not_called()
     mock_add_task.assert_called_once()  # Verify task was queued
 
 
+@patch("matchbox.server.api.main.BackgroundTasks.add_task")
+def test_upload_wrong_filetype(
+    mock_add_task: Mock,
+    s3: S3Client,
+    api_client_and_mocks: tuple[TestClient, Mock, Mock],
+):
+    """Test uploading a file that is not Parquet."""
+    # Setup
+    test_client, mock_backend, mock_tracker = api_client_and_mocks
+
+    mock_backend.settings.datastore.get_client.return_value = s3
+    mock_backend.settings.datastore.cache_bucket_name = "test-bucket"
+    mock_backend.insert_resolution = Mock(return_value=None)
+    mock_backend.insert_source_data = Mock(return_value=None)
+    s3.create_bucket(
+        Bucket="test-bucket",
+        CreateBucketConfiguration={"LocationConstraint": "eu-west-2"},
+    )
+
+    source_testkit = source_factory()
+
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
+
+    # Make request with mocked background task
+    response = test_client.post(
+        f"/upload/{update_id}",
+        files={
+            "file": (
+                "hashes.csv",
+                table_to_buffer(source_testkit.data),
+                "application/octet-stream",
+            ),
+        },
+    )
+
+    # Should fail when trying to read parquet table
+    assert response.status_code == 400
+    assert "server expected .parquet" in response.json()["details"].lower()
+    mock_add_task.assert_not_called()
+
+
+@patch("matchbox.server.api.main.BackgroundTasks.add_task")
+def test_upload_wrong_file_format(
+    mock_add_task: Mock,
+    s3: S3Client,
+    api_client_and_mocks: tuple[TestClient, Mock, Mock],
+):
+    """Test that file uploaded has Parquet magic bytes."""
+    # Setup
+    test_client, mock_backend, mock_tracker = api_client_and_mocks
+
+    mock_backend.settings.datastore.get_client.return_value = s3
+    mock_backend.settings.datastore.cache_bucket_name = "test-bucket"
+    mock_backend.insert_resolution = Mock(return_value=None)
+    mock_backend.insert_source_data = Mock(return_value=None)
+    s3.create_bucket(
+        Bucket="test-bucket",
+        CreateBucketConfiguration={"LocationConstraint": "eu-west-2"},
+    )
+
+    source_testkit = source_factory()
+
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
+
+    # Make request with mocked background task
+    response = test_client.post(
+        f"/upload/{update_id}",
+        files={
+            "file": (
+                "hashes.parquet",
+                b"dummy\ndata",
+                "application/octet-stream",
+            ),
+        },
+    )
+
+    # Should fail when trying to read parquet table
+    assert response.status_code == 400
+    assert "invalid parquet file" in response.json()["details"].lower()
+    mock_add_task.assert_not_called()
+
+
 # We can patch BackgroundTasks as the api_client_and_mocks fixture
 # ensures the API runs the task (not Celery)
 @patch("matchbox.server.api.main.BackgroundTasks.add_task")
@@ -126,7 +211,7 @@ def test_upload_wrong_schema(
 
     # Create source with results schema instead of index
     source_testkit = source_factory()
-    update_id = mock_tracker.add_source(source_testkit.source_config)
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
 
     response = test_client.post(
         f"/upload/{update_id}",
@@ -150,7 +235,7 @@ def test_upload_status_check(api_client_and_mocks: tuple[TestClient, Mock, Mock]
     """Test checking status of an upload using the status endpoint."""
     test_client, _, mock_tracker = api_client_and_mocks
     source_testkit = source_factory()
-    update_id = mock_tracker.add_source(source_testkit.source_config)
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
     mock_tracker.update(update_id, UploadStage.PROCESSING)
     mock_tracker.reset_mock()
 
@@ -166,12 +251,18 @@ def test_upload_already_processing(api_client_and_mocks: tuple[TestClient, Mock,
     """Test attempting to upload when status is already processing."""
     test_client, _, mock_tracker = api_client_and_mocks
     source_testkit = source_factory()
-    update_id = mock_tracker.add_source(source_testkit.source_config)
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
     mock_tracker.update(update_id, UploadStage.PROCESSING)
 
     response = test_client.post(
         f"/upload/{update_id}",
-        files={"file": ("test.parquet", b"dummy data", "application/octet-stream")},
+        files={
+            "file": (
+                "test.parquet",
+                table_to_buffer(source_testkit.data),
+                "application/octet-stream",
+            )
+        },
     )
 
     # Should return 400 with current status
@@ -183,12 +274,18 @@ def test_upload_already_queued(api_client_and_mocks: tuple[TestClient, Mock, Moc
     """Test attempting to upload when status is already queued."""
     test_client, _, mock_tracker = api_client_and_mocks
     source_testkit = source_factory()
-    update_id = mock_tracker.add_source(source_testkit.source_config)
+    update_id = mock_tracker.add_source(source_testkit.resolution_path)
     mock_tracker.update(update_id, UploadStage.QUEUED)
 
     response = test_client.post(
         f"/upload/{update_id}",
-        files={"file": ("test.parquet", b"dummy data", "application/octet-stream")},
+        files={
+            "file": (
+                "test.parquet",
+                table_to_buffer(source_testkit.data),
+                "application/octet-stream",
+            )
+        },
     )
 
     # Should return 400 with current status
@@ -225,38 +322,68 @@ def test_query(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
 
     response = test_client.get(
         "/query",
-        params={"source": "foo", "return_leaf_id": False},
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "source": "foo",
+            "return_leaf_id": False,
+        },
     )
 
+    assert response.status_code == 200
+
     buffer = BytesIO(response.content)
     table = pq.read_table(buffer)
 
-    assert response.status_code == 200
     assert table.schema.equals(SCHEMA_QUERY)
 
 
-def test_query_404_resolution(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
+def test_query_404(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     test_client, mock_backend, _ = api_client_and_mocks
+
+    mock_backend.query = Mock(side_effect=MatchboxCollectionNotFoundError())
+
+    response = test_client.get(
+        "/query",
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "source": "foo",
+            "return_leaf_id": True,
+        },
+    )
+
+    assert response.status_code == 404
+    assert response.json()["entity"] == BackendResourceType.COLLECTION
+
+    mock_backend.query = Mock(side_effect=MatchboxRunNotFoundError())
+
+    response = test_client.get(
+        "/query",
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "source": "foo",
+            "return_leaf_id": True,
+        },
+    )
+    assert response.status_code == 404
+    assert response.json()["entity"] == BackendResourceType.RUN
+
     mock_backend.query = Mock(side_effect=MatchboxResolutionNotFoundError())
 
     response = test_client.get(
         "/query",
-        params={"source": "foo", "resolution": "bar", "return_leaf_id": True},
-    )
-
-    assert response.status_code == 404
-
-
-def test_query_404_source(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
-    test_client, mock_backend, _ = api_client_and_mocks
-    mock_backend.query = Mock(side_effect=MatchboxSourceNotFoundError())
-
-    response = test_client.get(
-        "/query",
-        params={"source": "foo", "return_leaf_id": True},
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "source": "foo",
+            "return_leaf_id": True,
+        },
     )
 
     assert response.status_code == 404
+    assert response.json()["entity"] == BackendResourceType.RESOLUTION
 
 
 def test_match(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
@@ -264,9 +391,13 @@ def test_match(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     mock_matches = [
         Match(
             cluster=1,
-            source="foo",
+            source=SourceResolutionPath(
+                collection="test_collection", name="foo", run=1
+            ),
             source_id={"1"},
-            target="bar",
+            target=SourceResolutionPath(
+                collection="test_collection", name="bar", run=1
+            ),
             target_id={"a"},
         )
     ]
@@ -275,6 +406,8 @@ def test_match(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     response = test_client.get(
         "/match",
         params={
+            "collection": "test_collection",
+            "run_id": 1,
             "targets": "foo",
             "source": "bar",
             "key": 1,
@@ -287,13 +420,50 @@ def test_match(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     [Match.model_validate(m) for m in response.json()]
 
 
-def test_match_404_resolution(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
+def test_match_404(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
     test_client, mock_backend, _ = api_client_and_mocks
+
+    mock_backend.match = Mock(side_effect=MatchboxCollectionNotFoundError())
+
+    response = test_client.get(
+        "/match",
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "targets": ["foo"],
+            "source": "bar",
+            "key": 1,
+            "resolution": "res",
+        },
+    )
+
+    assert response.status_code == 404
+    assert response.json()["entity"] == BackendResourceType.COLLECTION
+
+    mock_backend.match = Mock(side_effect=MatchboxRunNotFoundError())
+
+    response = test_client.get(
+        "/match",
+        params={
+            "collection": "test_collection",
+            "run_id": 1,
+            "targets": ["foo"],
+            "source": "bar",
+            "key": 1,
+            "resolution": "res",
+        },
+    )
+
+    assert response.status_code == 404
+    assert response.json()["entity"] == BackendResourceType.RUN
+
     mock_backend.match = Mock(side_effect=MatchboxResolutionNotFoundError())
 
     response = test_client.get(
         "/match",
         params={
+            "collection": "test_collection",
+            "run_id": 1,
             "targets": ["foo"],
             "source": "bar",
             "key": 1,
@@ -305,24 +475,6 @@ def test_match_404_resolution(api_client_and_mocks: tuple[TestClient, Mock, Mock
     assert response.json()["entity"] == BackendResourceType.RESOLUTION
 
 
-def test_match_404_source(api_client_and_mocks: tuple[TestClient, Mock, Mock]):
-    test_client, mock_backend, _ = api_client_and_mocks
-    mock_backend.match = Mock(side_effect=MatchboxSourceNotFoundError())
-
-    response = test_client.get(
-        "/match",
-        params={
-            "targets": ["foo"],
-            "source": "bar",
-            "key": 1,
-            "resolution": "res",
-        },
-    )
-
-    assert response.status_code == 404
-    assert response.json()["entity"] == BackendResourceType.SOURCE
-
-
 # Admin
 
 
@@ -381,10 +533,9 @@ def test_api_key_authorisation(api_client_and_mocks: tuple[TestClient, Mock, Moc
     test_client, _, _ = api_client_and_mocks
     routes = [
         (test_client.post, "/upload/upload_id"),
-        (test_client.post, "/sources"),
-        (test_client.post, "/models"),
-        (test_client.patch, "/models/name/truth"),
-        (test_client.delete, "/resolutions/name"),
+        (test_client.post, "/collections/default/runs/1/resolutions/name"),
+        (test_client.patch, "/collections/default/runs/1/resolutions/name/truth"),
+        (test_client.delete, "/collections/default/runs/1/resolutions/name"),
         (test_client.delete, "/database"),
     ]
 
@@ -418,16 +569,3 @@ def test_api_key_authorisation(api_client_and_mocks: tuple[TestClient, Mock, Moc
         response = method(url)
         assert response.status_code == 401
         assert response.content == b'"JWT required but not provided."'
-
-
-def test_get_resolution_graph(
-    resolution_graph: ResolutionGraph,
-    api_client_and_mocks: tuple[TestClient, Mock, Mock],
-):
-    """Test the resolution graph report endpoint."""
-    test_client, mock_backend, _ = api_client_and_mocks
-    mock_backend.get_resolution_graph = Mock(return_value=resolution_graph)
-
-    response = test_client.get("/report/resolutions")
-    assert response.status_code == 200
-    assert ResolutionGraph.model_validate(response.json())
